#! /usr/bin/env python2.5

from os import system
from os.path import dirname, abspath, join
from glob import glob
import sys

from os.path import join
from traceback import format_exc
from urllib import urlopen, urlencode
from urlparse import urlsplit
from lxml.etree import parse, tostring

from bzv.fulltextextractor import FullTextExtractor, SruUpdateClient
from merescocore.framework import be

xPathToFulltextUrlMods = '//mods:mods/mods:location/mods:url/text()'
xPathToFulltextUrlDidl = "//oai:metadata/didl:DIDL/didl:Item/didl:Item/didl:Component/didl:Resource[@mimeType='application/pdf']/@ref"

fulltextRecordsQuery = '(fulltext.url.exists exact true) NOT (fulltext.item.status = done OR fulltext.item.status = failed)'

namespaces = {
    'mods': 'http://www.loc.gov/mods/v3',
    'srw': 'http://www.loc.gov/zing/srw/',
    'meta': 'http://meresco.com/namespace/harvester/meta',
    'didl': 'urn:mpeg:mpeg21:2002:02-DIDL-NS',
    'dii': 'urn:mpeg:mpeg21:2002:01-DII-NS',
    'oai': 'http://www.openarchives.org/OAI/2.0/',
    'rdf': 'http://www.w3.org/1999/02/22-rdf-syntax-ns#',
}

class UrlFetcher(object):
    def urlopen(self, url):
        yield urlopen(url).read()

def main(sruHost, sruPort, sruPath, updateHost, updatePort, updatePath):
    dna = be(
        (FullTextExtractor(),
            (UrlFetcher(), ),
            (SruUpdateClient(updateHost, updatePort, updatePath), ),
        ),
    )

    xmlNode = parse(urlopen("http://%s:%s%s?%s" % (sruHost, sruPort, sruPath, urlencode({
        'operation':'searchRetrieve',
        'version':'1.1',
        'query': fulltextRecordsQuery,
        'recordPacking': 'xml',
        'recordSchema':'meta',
        'x-recordSchema':'metadata'}))))

    for record in xmlNode.xpath('//srw:record', namespaces=namespaces):
        uploadId = ''.join( record.xpath('srw:recordData/meta:meta/meta:upload/meta:id/text()', namespaces=namespaces))
        fulltextUrlsMods = record.xpath('srw:extraRecordData/*'+xPathToFulltextUrlMods, namespaces=namespaces)
        fulltextUrlsDidl = record.xpath('srw:extraRecordData/*'+xPathToFulltextUrlDidl, namespaces=namespaces)
        fulltextUrls = []
        fulltextUrls.extend(fulltextUrlsMods)
        fulltextUrls.extend(fulltextUrlsDidl)
        print uploadId, fulltextUrls
        try:
            dna.addFullText(uploadId, fulltextUrls)
        except Exception, e:
            # FIXME
            print format_exc(limit=7)
        print "DONE"

if __name__ == '__main__':
    from sys import argv, stdout
    args = argv[1:]
    if len(args) != 6:
        print "Usage: pendingfulltext.py example.org 7080 /sru example.org 7071 /update"
        exit(1)

    sruHost, sruPort, sruPath, updateHost, updatePort, updatePath = args

    main(sruHost, sruPort, sruPath, updateHost, updatePort, updatePath)
