#! /usr/bin/env python2.5

from distutils.core import setup
from distutils.util import subst_vars
from distutils.dist import Distribution
from distutils.command.install import install
from distutils.command.install_data import install_data
from glob import glob

class MyDistribution(Distribution):
	def __init__(self, attrs):
		self.subst_files = None
		return Distribution.__init__(self, attrs)

	def has_subst_files(self):
		return self.subst_files and len(self.subst_files) > 0

class myinstall(install):
	def has_subst_files(self):
		return self.distribution.has_subst_files()

	sub_commands = install.sub_commands + [
			('install_subst',   has_subst_files),
		]

class install_subst(install_data):
	description = '"build" files that require substitution'

	def initialize_options(self):
		self.prefix = None
		self.exec_prefix = None
		self.home = None
		self.root = None
		self.purelibdir = None
		self.platlibdir = None
		self.headersdir = None
		self.libdir = None
		self.scriptsdir = None
		install_data.initialize_options(self)

	def finalize_options(self):
		install_data.finalize_options(self)
		self.set_undefined_options('install',
				('prefix', 'prefix'),
				('exec_prefix', 'exec_prefix'),
				('home', 'home'),
				('root', 'root'),
				('install_purelib', 'purelibdir'),
				('install_platlib', 'platlibdir'),
				('install_headers', 'headersdir'),
				('install_lib', 'libdir'),
				('install_scripts', 'scriptsdir'),
			)
		self.data_files = self.distribution.subst_files

	def copy_file(self, src, dst):
		prefix = self.prefix
		if(prefix == None):
			prefix = self.home
		(out, done) = install_data.copy_file(self, src, dst)
		print "applying substitutions to " + out
		i = open(src, 'r')
		o = open(out, 'w')
		for line in i:
			o.write(subst_vars(line, {
			        'prefix': prefix,
				'Id': '$Id',
				'URL': '$URL'
                        }))
		o.close()
		i.close()
		return (out, done)

setup(
	name = 'bzv',
	version = '7.1.0',
	maintainer = 'Universiteit van Tilburg',
	maintainer_email = 'meresco@uvt.nl',
	url = 'http://www.non-gnu.uvt.nl/bzv/',
	package_dir = {'bzv': 'src', 'bzvtest': 'test'},
	packages = ['bzv', 'bzv.fulltextextractor'],
	scripts = glob('bin/*'),
	data_files = [
		('share/bzv/html/static', glob('html/static/*')),
		('share/bzv/html/dynamic', glob('html/dynamic/*')),
		('share/bzv/xslt', glob('xslt/*')),
		('share/bzv/schemas', glob('schemas/*')),
	],
	subst_files = [
		('share/doc/bzv/example', glob('etc/*')),
	],
	distclass = MyDistribution,
	cmdclass = {'install': myinstall, 'install_subst': install_subst},
)
