from merescocore.framework import Observable

class CheckPart(Observable):
    """
    CheckPart is used in the context of the Rss class.
    Th Rss class has a method that calls any.getRecord(recordId).
    getRecord is a method of XmlCompose, a super class of RssItem.
    A RssItem object defines the part that is used for a rss item.
    In our case this is sometimes a mods part and in others cases the metadata part.
    CheckPart is a mechanism to select the right RssItem object by checking
    whether the mods part is available. This is done by intercepting the
    getRecord call of Rss by implementing the getRecord method.
    Dependent on the context Checkpart.getRecord calls getRecord of
    XmlCompose or it calls useOtherPart which is another method of CheckPart.
    By including CheckPart within CheckPart, each with its own RssItem object,
    useOtherPart can be catched. In this way another RssItem definition can be
    used.
    
    The fact that the CheckPart methods getRecord() and useOtherPart() are calling each other 
    is not very elegant because of the danger of looping
    However this is only a theoretical danger because in the DNA there
    are only a finite number of observers.
    """

    def __init__(self, parts=[], action=''):
        Observable.__init__(self)
        assert len(parts) != 0 and ( action == 'continue' or action == 'stop'), 'Use parts and action="continue|stop"'
        self._parts = parts
        self._action = action

    def getRecord(self, recordId):
        Continue = True 
        for part in self._parts:
            ignored, partExists = self.any.isAvailable(recordId, part)
            if partExists:
                if self._action == 'stop':
                    Continue = False
                    break
            else:
                if self._action == 'continue':
                    Continue = False
                    break
        if Continue:
            return self.any.getRecord(recordId)
        else:
            return self.any.useOtherPart(recordId)

    def useOtherPart(self, recordId):
            return self.any.getRecord(recordId)
