import os
import ConfigParser
from traceback import format_exc
from os.path import isfile
import sys

class CustomerConfigException (Exception):
      def __init__(self, value):
            self.value = value

      def __str__(self):
            return repr(self.value)

class CustomerConfig:
      """
      Provide simple configuration file support, for multiple languages (default: 'en' i.e. English).
      The code uses the standard python configparser:  http://docs.python.org/library/configparser.html
      The format of the configfile is thus presribed by the configparser.

      The most important call is:

      get(option, section, default)

      The value returned is:
        1 The value actually found. If not persent:
        2 The value of the 'default' parameter. If not present:
        3 The content of the 'option' parameter. If not present:
        4 None, raise CustomerConfigException
      """
      def __init__(self, configFile, infra, language = 'en', section = ''):
            self.infra      = infra
            self.logging    = infra.getLoggingInstance()
            self.logger     = self.logging.getLogger('meresco.configs')

            self.configFile = configFile
            #FIXME Enlish rules!
            self.language   = 'en' #language
            self.section    = section
            self.configParser     = ConfigParser.ConfigParser()
            if not isfile(configFile):
               self.logger.error("Configfile NOT FOUND: " + configFile)
            else:
               self.configParser.read(configFile)
            self.logger.debug('Init for customerConfig, file: '+ configFile)

      def setDefaults(self, section, language = 'en'):
            self.section = section
            self.language= 'en'

      def setSection(self, section):
            self.section = section

      def getConfig(self):
            return self.configParser

      def setLanguage(self, language):
            self.language = 'en'

      def setSubst(self, substDict):
            self.substDict = substDict

      def get(self, option, default = '', section = '',substDict={}):
            config  = self.configParser
            logger  = self.logger
            sane = True
            value =''

            if (substDict == ''): substDict=self.substDict
            if (section == ''): section = self.section

            if (option == None or option == ''):
                  logger.error("No valid option specified")
                  sane = False

            if (section == None or section == '') :
                  logger.error("No section specified")
                  sane = False

            if (default != '' and default != None ):
                  value = default
            elif (option != '' and option != None ):
                  value = option

            if (sane):
                  try:
                        value = config.get(section, option, 0, substDict)
                  except:
                        logger.debug (format_exc())
                        try:
                              value = config.get(section, option, 0, substDict)
                        except Exception, e:
                              #logger.warning (format_exc()), prints the customerConfig stack only :(
                              logger.warning ('"' + option + '[_' + self.language + ']'  +
                                              '" NOT specified ' + 'in file: "' +
                                              self.configFile + '" section: "' + section + '". Using default value: ' + default)
                        #     raise CustomerConfigException("Request not met")

            else:
                   raise CustomerConfigException("Incomplete request")

            if (value):
                  logger.debug('returning value: [' + value +']')

            return value

      def dump(self):
            s = ''
            for section in config.sections():
                  s = s + ("section: " +  section )
                  for option in config.options(section):
                        s = s + " " + option + "=" + config.get(section, option) + "\n"
            return s




