from re import compile

monthTable = [
    None, '01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12',
    None, None, None, None, None, None, None, None,
    '03', '06', '09', '12',  # Elsevier uses these for Spring, Summer, Autumn, Winter
    None, None, None, None, None, None,
    '01', '04', '07', '10'   # Elsevier uses these for Q1, Q2, Q3, Q4
]

def createDateIssued(dtString):
    dateRange = parseDateRange(dtString)
    if dateRange == None:
        raise ValueError("'%s' is not a valid _dt value" % dtString)
    if len(dateRange) == 2:
        return '<dateIssued qualifier="inferred" encoding="iso8601" point="start">%s</dateIssued><dateIssued qualifier="inferred" encoding="iso8601" point="end">%s</dateIssued>' % dateRange
    return '<dateIssued qualifier="inferred" encoding="iso8601">%s</dateIssued>' % dateRange


regexps = [
    (compile(r'^(?P<year>\d{4})$'),
        '%(year)s'),
    (compile(r'^(?P<year>\d{4})(?P<month>\d{2})$'),
        '%(year)s-%(month)s'),
    (compile(r'^(?P<year>\d{4})(?P<month>\d{2})(?P<day>\d{2})$'),
        '%(year)s-%(month)s-%(day)s'),
    (compile(r'^(?P<year>\d{4})(?P<month1>\d{2})/(?P<month2>\d{2})$'),
        '%(year)s-%(month1)s', '%(year)s-%(month2)s'),
    (compile(r'^(?P<year1>\d{4})(?P<month1>\d{2})/(?P<year2>\d{4})(?P<month2>\d{2})$'),
        '%(year1)s-%(month1)s', '%(year2)s-%(month2)s'),
    (compile(r'^(?P<year>\d{4})(?P<month>\d{2})(?P<day1>\d{2})/(?P<day2>\d{2})$'),
        '%(year)s-%(month)s-%(day1)s','%(year)s-%(month)s-%(day2)s'),
    (compile(r'^(?P<year>\d{4})(?P<month1>\d{2})(?P<day1>\d{2})/(?P<month2>\d{2})(?P<day2>\d{2})$'),
        '%(year)s-%(month1)s-%(day1)s','%(year)s-%(month2)s-%(day2)s'),
    (compile(r'^(?P<year1>\d{4})/(?P<year2>\d{4})$'),
        '%(year1)s', '%(year2)s'),
    (compile(r'^(?P<year>\d{4})/(?P<month>\d{2})$'),
        '%(year)s-%(month)s'),
    (compile(r'^(?P<year>\d{4})(?P<month1>\d{2})/(?P<month2>\d{2})(?P<day>\d{2})$'),
        '%(year)s-%(month1)s', '%(year)s-%(month2)s-%(day)s'),        
    (compile(r'^(?P<year1>\d{4})(?P<month1>\d{2})(?P<day1>\d{2})/(?P<year2>\d{4})(?P<month2>\d{2})(?P<day2>\d{2})$'),
        '%(year1)s-%(month1)s-%(day1)s', '%(year2)s-%(month2)s-%(day2)s'),
]

def parseDateRange(dtString):
    for regexpAndTemplates in regexps:
        regexp = regexpAndTemplates[0]
        match = regexp.match(dtString)
        if match:
            dateInfo = match.groupdict()
            if 'month' in dateInfo:
                dateInfo['month'] = monthTable[int(dateInfo['month'])]
            return tuple(template % dateInfo for template in regexpAndTemplates[1:])