from merescocore.framework import Observable
from xml.sax.saxutils import escape as escapeXml
from subprocess import Popen, PIPE
from StringIO import StringIO
from string import maketrans
from tempfile import NamedTemporaryFile
from time import strftime, gmtime

from magic import open as open_magic, MAGIC_MIME
magicCookie = open_magic(MAGIC_MIME)
magicCookie.load()


RESPONSE_HEADER = """<fulltext xmlns="http://meresco.com/namespace/fulltext">
    <identifier>%(identifier)s</identifier>
    <datestamp>%(datestamp)s</datestamp>
"""
RESPONSE_ITEM = """
    <item>
        <url>%(url)s</url>
        <status>%(status)s</status>
        <errorCode>%(errorCode)s</errorCode>
        <data>%(data)s</data>
    </item>
"""
RESPONSE_FOOTER = """
</fulltext>
"""

class NotPDF(Exception):
    def __str__(self):
        return "Not a PDF"

class FullTextExtractor(Observable):
    def _isPdf(self, aFilename):
        return magicCookie.file(aFilename) == 'application/pdf'

    def addFullText(self, identifier=None, urls=None):
        recordData = ""
        recordData += RESPONSE_HEADER % {
            'identifier': escapeXml(identifier),
            'datestamp': self._datestamp(),
        }
        for url in urls:
            try:
                f = NamedTemporaryFile()
                for stuff in self.any.urlopen(url):
                    f.write(stuff)
                f.seek(0L)
                if not self._isPdf(f.name):
                    raise NotPDF()
                data = self._pdfToText(f.name)
                status = 'done'
                errorCode = ''
            except Exception, e:
                data = ''
                status = 'failed'
                errorCode = str(e)
            finally:
                f.close()
            recordData += RESPONSE_ITEM % {
                'url': escapeXml(url),
                'status': escapeXml(status),
                'errorCode': escapeXml(errorCode),
                'data': escapeXml(data),
                }
        recordData += RESPONSE_FOOTER
#        f = open('/tmp/'+identifier+'.txt', 'w')
#        f.write(recordData)
#        f.close()
        self.any.sendReplace(identifier, recordSchema='fulltext', recordData=recordData)


    def _pdfToText(self, filename):
        outfile = NamedTemporaryFile()
        devnull = open('/dev/null', 'rw')
        try:
            pipe = Popen(args=['pdftotext', filename, outfile.name], stdin=devnull, stdout=PIPE, stderr=devnull)
            out, errs = pipe.communicate()
            if errs:
                raise Exception(errs)
            trin = "\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f"
            trout = "                                "
            result = ' '.join(outfile.read().strip().split()).translate(maketrans(trin, trout), '')
        finally:
            outfile.close()

        return result

    def _datestamp(self):
        return strftime('%Y-%m-%dT%H:%M:%SZ', gmtime())
