from xml.sax.saxutils import escape as escapeXml
from urllib import urlopen
from lxml.etree import parse, tostring

NAMESPACES = {
    'srw': 'http://www.loc.gov/zing/srw/',
    'ucp': 'info:lc/xmlns/update-v1',
    'diag': 'http://www.loc.gov/zing/srw/diagnostic/'
}

class SruUpdateClientException(Exception):
    pass

class SruUpdateClient(object):
    def __init__(self, host, port, path):
        self._host = host
        self._port = port
        self._path = path

    def sendReplace(self, recordIdentifier, recordPacking="xml", recordSchema="", recordData=""):
        recordIdentifier = escapeXml(recordIdentifier)
        recordUpdate = """<?xml version="1.0" encoding="UTF-8"?>
<updateRequest xmlns:srw="http://www.loc.gov/zing/srw/" xmlns:ucp="http://www.loc.gov/ucp">
    <srw:version>1.0</srw:version>
    <ucp:action>info:srw/action/1/replace</ucp:action>
    <ucp:recordIdentifier>%(recordIdentifier)s</ucp:recordIdentifier>
    <srw:record>
        <srw:recordPacking>%(recordPacking)s</srw:recordPacking>
        <srw:recordSchema>%(recordSchema)s</srw:recordSchema>
        <srw:recordData>%(recordData)s</srw:recordData>
    </srw:record>
</updateRequest>""" % locals()

        result = self._sendRequest("http://%s:%s%s" % (self._host, self._port, self._path), recordUpdate)

    def _sendRequest(self, url, data):
        responseStream = self._urlopen(url, data)
        xml = parse(responseStream)
        operationStatus = xml.xpath("/srw:updateResponse/ucp:operationStatus/text()", namespaces=NAMESPACES)[0]

        if operationStatus != 'success':
            raise SruUpdateClientException(tostring(xml))

    def _urlopen(self, url, data):
        return urlopen(url, data=data)


