from cqlparser.cqlparser import findLastScopedClause, parseString, BOOLEAN, CQLParseException
from cqlwebutils import parseCqlFromWeb, Query, CQLQueryPrinter

def createCqlQuery(query, drilldownArguments, extra):
    q = parseCqlFromWeb(query)
    # Anton: s/true/false zodat een lege string niets oplevert,
    #        is dit een geschikte plek hiervoor?
    if q.isEmpty():
        q.add('meresco.exists')
        q.add('false')
        q.EXACT()
    q = addAndExactArguments(q, drilldownArguments)
    if extra:
        q = addAndExactArguments(q, extra)
    return str(q)

def addAndExactArguments(query, arguments):
    needsExtraAnd = False
    if query == None:
        query = Query()
        query.setPrinterClass(CQLQueryPrinter)
    else:
        needsExtraAnd = not query.isEmpty()

    for fieldAndTerm in arguments:
        field, term = fieldAndTerm.split(':', 1)
        query.add(field)
        query.add(term)
        query.EXACT()

    for i in range(1, len(arguments)):
        query.AND()

    if needsExtraAnd and len(arguments) > 0:
        query.AND()
    return query

def andQuery(operand1, operand2):
    lastScopedClause = findLastScopedClause(operand1._children[-1])
    lastScopedClause._children += (BOOLEAN('and'),) + operand2._children
    return operand1

def googleLikeParseTree(httpQuery, drilldownArguments, extra):
    cqlQuery = None
    if drilldownArguments:
        cqlQuery = addAndExactArguments(cqlQuery, drilldownArguments)

    if extra:
        cqlQuery = addAndExactArguments(cqlQuery, extra)

    cqlQuery = cqlQuery != None and str(cqlQuery) or ''

    parsetree = None
    cqlToPassAlong = ''
    try:
        ungoogledQuery = unGoogleQuery(httpQuery, antiUnaryClause='meresco.exists exact true')
        parsetree = parseString("(" + ungoogledQuery + ")")

        cqlToPassAlong = ungoogledQuery
        if cqlQuery:
#            parsetree = parsetree.andQuery(parseString(cqlQuery))
            parsetree = andQuery(parsetree, parseString(cqlQuery))
            cqlToPassAlong = "(" + ungoogledQuery + ") AND " + str(cqlQuery)

    except CQLParseException, e:
        pass

    return parsetree, cqlToPassAlong

def unGoogleQuery(aString, antiUnaryClause=""):
    source = aString.replace('(', '( ')
    parts = source.split()
    for index, part in enumerate(parts):
        if part[0] =='-' and len(part) > 1:
            parts[index] = parts[index][1:]
            parts.insert(index, 'NOT')
            if index == 0 or parts[index-1] == '(':
                parts.insert(index, antiUnaryClause)
    return ' '.join(parts).replace('( ', '(')

def isGoogleLikeQuery(aString):
    googleLike = False
    for part in aString.lower().split():
        if part in ['and', 'or', 'not']:
            googleLike = True
        elif part[0] in ['(', '-'] or part[-1] == ')':
            googleLike = True

    return googleLike

def isValidGoogleQuery(googleQuery):
    try:
        parseString(unGoogleQuery(googleQuery, antiUnaryClause='meresco.exists exact true'))
        return True
    except CQLParseException, e:
        return False

class GoogleLikeQuery(object):

    def parseQuery(self, httpQuery, drilldownArguments, extra=[]):
        parsetree = None
        if isGoogleLikeQuery(httpQuery):
            parsetree, cqlQuery = googleLikeParseTree(httpQuery, drilldownArguments, extra)

        if not parsetree:
            cqlQuery = createCqlQuery(httpQuery, drilldownArguments, extra)
            parsetree = parseString(cqlQuery)

        return parsetree, cqlQuery

    def isGoogleLikeQuery(self, aString):
        return isGoogleLikeQuery(aString)

    def isValidGoogleQuery(self, aString):
        return isValidGoogleQuery(aString)
