from bzv.yearnormalize import YearNormalize
from urllib import urlencode
from re import compile

# mapping to OpenURL genre
genreMap = {
    'bookPart'                : 'bookitem',
    'bookReview'              : 'article',
    'conferencePaper'         : 'proceeding',
    'contributionToPeriodical': 'article',
    'workingPaper'            : 'preprint'
}

urnRe = compile(r'^(?i)urn:(\w+):([\w-]+)$')

def xpath(node, path):
    return node.xpath(path, {'mods':'http://www.loc.gov/mods/v3'})
def xpathString(node, path):
    return ''.join(xpath(node, path)[:1])

def getURN(identifierString):
    m = urnRe.match(identifierString)
    if not m:
        return '', ''
    type, identifier = m.groups()
    return type.lower(), identifier

def add(aDictionary, key, value):
    if value:
        aDictionary[key] = value
    
class OpenURL(object):
    def __init__(self, baseurl):
        self._baseurl = baseurl

    def createOpenURL(self, lxmlNode):
        return '%s?%s' % (self._baseurl, self._create(lxmlNode))
        
    def _create(self, lxmlNode):
        data = {}
        genre = xpathString(lxmlNode, 'mods:genre/text()').split('/')[-1]
        openurlGenre = genre
        if genreMap.has_key(genre): openurlGenre = genreMap[genre]
        add(data, 'genre', openurlGenre)
        add(data, 'aulast', xpathString(lxmlNode, 'mods:name[@type="personal"][mods:role/mods:roleTerm="aut"]/mods:namePart[@type="family"]/text()'))
        add(data, 'aufirst', xpathString(lxmlNode, 'mods:name[@type="personal"][mods:role/mods:roleTerm="aut"]/mods:namePart[@type="given"]/text()').rstrip('.'))
        title = xpathString(lxmlNode, 'mods:titleInfo/mods:title/text()')
        relatedTitle = xpathString(lxmlNode, 'mods:relatedItem/mods:titleInfo/mods:title/text()')
        if genre in ['article', 'workingPaper', 'conferencePaper', 'contributionToPeriodical', 'bookPart', 'bookReview'] or (not genre and relatedTitle):
            add(data, 'atitle', title)
            add(data, 'title', relatedTitle)
        else:
            add(data, 'title', title)
#        identifier = xpathString(lxmlNode, '//mods:identifier/text()')
        identifiers = xpath(lxmlNode, '//mods:identifier/text()')
        for identifier in identifiers:
            typeIdentifier, identifier = getURN(identifier)
            if typeIdentifier in ['issn', 'isbn']:
                add(data, typeIdentifier, identifier.replace('-',''))
        volume = xpathString(lxmlNode, 'mods:relatedItem/mods:part/mods:detail[@type="volume"]/mods:number/text()')
	add(data, 'volume', volume.split('-')[0])
        issue = xpathString(lxmlNode, 'mods:relatedItem/mods:part/mods:detail[@type="issue"]/mods:number/text()')
	add(data, 'issue', issue.split('-')[0])
        spage = xpathString(lxmlNode, 'mods:relatedItem/mods:part/mods:extent[@unit="page"]/mods:start/text()')
        epage = xpathString(lxmlNode, 'mods:relatedItem/mods:part/mods:extent[@unit="page"]/mods:end/text()')
        add(data, 'spage', spage)
        add(data, 'epage', epage)
        if spage and epage:
            add(data, 'pages', '%s-%s' % (spage, epage))
        add(data, 'date',  YearNormalize(yearRange=(1000,2100)).normalize(xpathString(lxmlNode, 'mods:originInfo/mods:dateIssued/text()')))
#        add(data, 'location', xpathString(lxmlNode, 'mods:location/mods:holdingSimple/mods:copyInformation/mods:shelfLocator/text()'))

        #'auinit'
        #'auinit1'
        #'auinitm'
        #'eissn'
        #'coden'
        #'sici'
        #'bici'
        #'part'
        
        #'artnum'
        #'date'
        #'ssn'
        #'quarter'

        return urlencode(data)
