from cgi import parse_qs
from lxml.etree import parse
from merescocore.components.http import utils as httputils
from merescocore.framework import Observable
from urlparse import urlsplit
from xml.sax.saxutils import escape
from bzv.jsonencoder import JSONEncoder

namespaceMap = {
    'didl' :"urn:mpeg:mpeg21:2002:02-DIDL-NS",
    'dip'  :"urn:mpeg:mpeg21:2005:01-DIP-NS",
    'mods' :"http://www.loc.gov/mods/v3",
    'rdf'  :"http://www.w3.org/1999/02/22-rdf-syntax-ns#",
    'extra':"http://meresco.com/namespace/fields/extra",
    'meta' :"http://meresco.com/namespace/harvester/meta",
}

def okJSON():
    CRLF = "\r\n"
    ContentTypeJSON = "Content-Type: application/json"
    Ok = "HTTP/1.0 200 Ok" + CRLF
    okJSON = "HTTP/1.0 200 OK" + CRLF + \
             ContentTypeJSON + CRLF + \
             CRLF
    return okJSON

def xpath(node, path):
    #FIXME: what if node == None
    result = node.xpath(path, namespaceMap)
    return result and result[0] or ''

def nodeset(node, path):
    #FIXME: what if node == None
    return node.xpath(path, namespaceMap)

def add(aDictionary, key, value):
    if value:
            aDictionary[key] = value

def makeUrlContent(url):
    u = '<img src="/files/edoc.gif" alt="*" border="0"/> <a target="_blank" href="%(url)s">%(url)s</a>' %{'url': escape(url)}
    return u

class Record(Observable):
    def __init__(self, infra = None):
         Observable.__init__(self )
         self.infra = infra
         self.infra.logging.debug('Starting record logging')
         self.logger = infra.logging.getLogger('meresco.record')
         self.logger.debug('Starting record logger')

    def setMetadata(self, recordId):
        try:
            if recordId.split(':')[0] in ['opc-uvt-nl']:
                #### gebruik de volgende regel voor de proto database op pukurin
                #  self._metadata = parse(self.any.getStream(recordId, 'metadata'))
                #### gebruik deze regel in alle andere gevallen
                self._metadata = parse(self.any.getStream(recordId, 'mods'))
            else:
                self._metadata = parse(self.any.getStream(recordId, 'metadata'))
#            self.infra.logging.debug(self.infra.prpt("----METADATA---",self._metadata))

        except Exception:
            self.logger.error("No metadata for record: " + recordId)
            self._metadata = None

    def getSid(self, recordId):
        self.logger.debug("recordId: " + recordId)

        meta = parse(self.any.getStream(recordId, 'meta'))
        collection = xpath(meta, '/meta:meta/meta:repository/meta:collection/text()')
        db = xpath(meta, '/meta:meta/meta:repository/meta:repositoryGroupId/text()')
        return collection + ':' + db

    def setGenre(self, recordId):
        genre = xpath(self._metadata, '//mods:mods/mods:genre/text()')
        if genre and '/' in genre: genre = genre.split('/')[-1]
####### The following will become redundant because in the future there will be 
####### no journals with genre == 'series' 
        if genre == 'series' and recordId.split(':')[0] in ['opc-uvt-nl']:
            journal = 0
            locations = nodeset(self._metadata, '//mods:mods/mods:location/mods:holdingSimple/mods:copyInformation/mods:shelfLocator')
            for loc in locations:
                t = xpath(loc, 'text()').split()[0]
                if t in ['T', 'E', 'TF']: journal = 1
            if journal: genre = "journal"
        self._genre = genre

    def getMetadata(self, recordId):
       return self._metadata

    def getOrder(self):
###### Note that all fields must be listed here
######      otherwise they will not be displayed.
###### This is a public method.
       order = {\
           1: 'Title',\
           2: 'Language',\
           3: 'Author(s)',\
           4: 'Collaborator(s)',\
           5: 'Editor(s)',\
           6: 'Supervisor(s)',\
           7: 'Corporation',\
           8: 'Source',\
           9: 'Publication date',\
           10: 'URL(s)',\
           11: 'Abstract',\
           12: 'Keyword(s)',\
           13: 'Classification',\
           14: 'ISSN',\
           15: 'ISBN',\
           16: 'Publisher',\
           17: 'Extent',\
           18: 'Series',\
           19: 'Note(s)',\
           20: 'Reference',\
           21: 'Publication type',\
           22: 'Location',\
           23: 'Related Titles'
       }
       return order

    def createRecord(self, recordId):
        self.setMetadata(recordId)

        try:
            self.setGenre(recordId)
        except Exception:
            self._genre=''
            self.logger.error("no Genre for " + recordId)

        self.record = {}
        add(self.record, 'Title', self._getTitle())
        add(self.record, 'Language', self._getLanguage())
        self._handleNames()
        add(self.record, 'Source', self._getSource())
        add(self.record, 'Publication date', self._getDate())
        add(self.record, 'URL(s)', self._getUrls())
        add(self.record, 'Abstract', self._getAbstract())
        add(self.record, 'Keyword(s)', self._getKeywords())
        add(self.record, 'Classification', self._getClassification())
        self._handleIdentifiers()
        add(self.record, 'Publisher', self._getPublisher())
        add(self.record, 'Publication type', self._genre)
        add(self.record, 'Extent', self._getExtent())
        add(self.record, 'Series', self._getSeries(recordId))
        add(self.record, 'Note(s)', self._getNotes())
        add(self.record, 'Reference', self._getReference())
        add(self.record, 'Location', self._getLocation())
        add(self.record, 'Related Titles', self._getRelatedTitles(recordId))
        return self.record

    def getMods(self, recordId):
        self.setMetadata(recordId)
        return xpath(self._metadata, '//mods:mods')

    def _addName(self, names, name):
        family = ""
        given  = ""
        infix  = ""
        for namePart in nodeset(name, 'mods:namePart'):
            type = xpath(namePart, '@type')
            if type == 'family' or not type:
                f = xpath(namePart, 'text()').split(',', 1)
                family = f[0]
                if len(f) == 2: infix  = f[1]
            elif type == 'given':  given  = xpath(namePart, 'text()')
        if family: names += family
        if given:
            names  += ', ' + given
            if infix: names += ' ' + infix
        if family: names += '; '
        return names

    def _getDate(self):
        date = xpath(self._metadata, '//mods:originInfo/mods:dateIssued/text()')
        if date:
            altDate = date.split('(', 1)
            if len(altDate) == 2: return altDate[1].rstrip(')')
            else: return date

    def _getLocation(self):
        locations = nodeset(self._metadata, '//mods:mods/mods:location/mods:holdingSimple/mods:copyInformation/mods:shelfLocator')
        if locations:
            s = ""
            for loc in locations:
                s += xpath(loc, 'text()') + "; "
            return s.strip('; ')

    def _getRelatedTitles(self, recordId):
        if self._genre == 'series':
            parts = recordId.split(':')
            if parts and parts[0] == 'opc-uvt-nl':
                return '<a href="/?query=%s+%s ">click here for the items in the series</a>' %(parts[0], parts[1])

    def _getSeries(self, recordId):
        out = ""
        series = (nodeset(self._metadata, '//mods:relatedItem[@type="series"]'))
        if series:
            for aseries in series:
                title = escape(xpath(aseries, 'mods:titleInfo/mods:title/text()'))
                ppn   = xpath(aseries, 'mods:recordInfo/mods:recordIdentifier/text()')
                part  = xpath(aseries, 'mods:part/mods:detail/mods:number/text()')
                if ppn:
                    out += '<a href="/?query=%s+%s">%s</a>' %(recordId.split(':')[0], ppn, title)
                else:
                    out += title
                if part: out += '<i>: ' + part + '</i>'
                out += '; '
            return out.rstrip('; ')

    def _getSource(self):
        editors = escape(self._getHostEditors())
	title = escape(self._getHostTitle())
        if not title: return ''
	if editors: return editors + ' ' + '<i>' + title + '</i>'
        date = self._getDate()
        volume = xpath(self._metadata, '//mods:part/mods:detail[@type="volume"]/mods:number/text()')
        issue = xpath(self._metadata, '//mods:part/mods:detail[@type="issue"]/mods:number/text()')
        firstpage = xpath(self._metadata, '//mods:part/mods:extent/mods:start/text()')
        endpage = xpath(self._metadata, '//mods:part/mods:extent/mods:end/text()')
        if volume: title += ' vol. ' + volume
        if date: title += ' (' + date + ')'
        if issue: title += ' nr. ' + issue
        if firstpage:
            title += ' p.' + firstpage
            if endpage: title += '-' + endpage
	part = xpath(self._metadata, '//mods:relatedItem[@type="host"]/mods:part/mods:text/text()')
        if part: title += '. ' + part
	return title

    def _getTitle(self):
        title = xpath(self._metadata, '//mods:mods/mods:titleInfo/mods:title/text()')
        if not title:
            title = xpath(self._metadata, '//mods:relatedItem[@type="host"]/mods:titleInfo/mods:title/text()')
            if title: title = '[' + title + ']'
            #fixme: naar config
            else: title = '[NO TITLE AVAILABLE]'
        subTitles = nodeset(self._metadata, '//mods:mods/mods:titleInfo/mods:subTitle/text()')
        if subTitles:
            for subTitle in subTitles:
                title += ": " + subTitle
        return escape(title)

    def _handleNames(self):
        authors = ""
        collaborators = ""
        editors = ""
        supervisors = ""
	names = (nodeset(self._metadata, '//mods:mods/mods:name'))
	if not names: return
        for name in names:
            role = xpath(name, 'mods:role/mods:roleTerm/text()')
            if   role == 'aut' or not role:
                authors     = self._addName(authors,     name)
            elif role == 'clb':
                collaborators = self._addName(collaborators, name)
            elif role == 'edt':
                editors     = self._addName(editors,     name)
            elif role == 'ths':
                supervisors = self._addName(supervisors, name)
            elif role == 'dgg':
                content = 'Degree granted by ' + str(xpath(name, 'mods:namePart/text()'))
                add(self.record, 'Corporation',   content)
            if authors:
                add(self.record, 'Author(s)',     authors.rstrip(' ;'))
            if collaborators:
                add(self.record, 'Collaborator(s)', collaborators.rstrip(' ;'))
            if editors:
                add(self.record, 'Editor(s)',     editors.rstrip(' ;'))
            if supervisors:
                add(self.record, 'Supervisor(s)', supervisors.rstrip(' ;'))

    def _getExtent(self):
        return xpath(self._metadata, '//mods:mods/mods:physicalDescription/mods:extent/text()')

    def _getLanguage(self):
        return xpath(self._metadata, '//mods:mods/mods:language/mods:languageTerm/text()')

    def _getHostTitle(self):
        return xpath(self._metadata, '//mods:mods/mods:relatedItem[@type="host"]/mods:titleInfo/mods:title/text()')

    def _getHostEditors(self):
        editors = ""
	family = ""
	given = ""
        for name in nodeset(self._metadata, '//mods:mods/mods:relatedItem/mods:name'):
            if xpath(name, 'mods:role/mods:roleTerm/text()') == 'edt':
                for namePart in nodeset(name, 'mods:namePart'):
                    type = xpath(namePart, '@type')
                    if type == 'family': family = xpath(namePart, 'text()')
                    elif type == 'given':  given = xpath(namePart, 'text()')
                if family: editors += family
                if given:  editors += ', ' + given
		if family: editors += '; '
		family = ""
		given = ""
        if editors: return editors.rstrip('; ') + ' (Ed.)'
	else: return ''

    def _getAbstract(self):
        return escape(xpath(self._metadata, '//mods:abstract/text()'))

    def _getUrls(self):
        urls = []
        url = xpath(self._metadata, '//mods:location/mods:url/text()')
        if url: urls.append(url)
        else:
            url = xpath(self._metadata, '//mods:location/mods:holdingSimple/mods:copyInformation/mods:electronicLocator/text()')
            if url: urls.append(url)
        for Item in nodeset(self._metadata, '//didl:DIDL/didl:Item/didl:Item'):
             if xpath(Item, 'didl:Descriptor/didl:Statement/rdf:type/text()') == 'info:eu-repo/semantics/objectFile' or xpath(Item, 'didl:Descriptor/didl:Statement/dip:ObjectType/text()') == 'info:eu-repo/semantics/objectFile':
                 url = xpath(Item, 'didl:Component/didl:Resource/@ref')
                 if url: urls.append(url)
        return urls

    def _getNotes(self):
        notes = ""
        count = 0
        for note in nodeset(self._metadata, '//mods:mods/mods:note/text()'):
             count += 1
             if count > 1:
                 notes += ". "
             notes += escape(note)
        return notes

    def _getReference(self):
        return xpath(self._metadata, '//mods:mods/mods:relatedItem[@type="isReferencedBy"]/mods:titleInfo/mods:title/text()')

    def _getPublisher(self):
        place = xpath(self._metadata, '//mods:mods/mods:originInfo/mods:place/mods:placeTerm/text()')
        name = xpath(self._metadata, '//mods:mods/mods:originInfo/mods:publisher/text()')
        publisher = ""
        if name:
           publisher = ""
           if place: publisher = place + ": "
           publisher += name
        return escape(publisher)

    def _getKeywords(self):
        keywords = ""
        for topic in nodeset(self._metadata, '//mods:mods/mods:subject/mods:topic'):
            keywords += xpath(topic, 'text()') + '; '
	if keywords: return keywords.rstrip('; ')
	else: return ''

    def _getClassification(self):
        string = ""
	for classification in nodeset(self._metadata, '//mods:mods/mods:classification'):
            string += xpath(classification, 'text()') + '; '
	return string.rstrip('; ')

    def _handleIdentifiers(self):
	identifiers = (nodeset(self._metadata, '//mods:identifier'))
	if not identifiers: return
        issns = ""
        isbns = ""
	for id in identifiers:
            idParts = xpath(id, 'text()').split(':')
            ### urn:{issn|issbn}:<value>
            if len(idParts) < 3: break
            if idParts[1] == 'issn' or idParts[1] == 'ISSN':
                if issns: issns += ", "
                issns += idParts[2]
            if idParts[1] == 'isbn' or idParts[1] == 'ISBN':
                if isbns: isbns += "; "
                displayLabel = xpath(id, '@displayLabel')
                if displayLabel: isbns += displayLabel
                else: isbns += idParts[2]
            if issns: add(self.record, 'ISSN', issns)
            if isbns: add(self.record, 'ISBN', isbns)

class RecordXml(Observable):
    def __init__(self):
         Observable.__init__(self)

    def handleRequest(self, RequestURI='', **kwargs):
####### TP: toevoegen het afvangen van exceptions: 1. er is geen record id,
####### 2. er wordt geen metadata gevonden
        Scheme, Netloc, Path, Query, Fragment = urlsplit(RequestURI)
        arguments = parse_qs(Query)
        recordId = arguments.get('recordId', [''])[0]
        record = self.any.createRecord(recordId)
        yield httputils.okXml
        yield """<?xml version="1.0" encoding="UTF-8"?>
<record>"""

####### TP: labels zijn in het Engels. Toevoegen: ondersteunen van andere talen.
####### TP: ook handig is om per field het type toe te voegen (taalonafhankelijk)
        order = self.any.getOrder()
        order.keys().sort()
        for key in order.keys():
            label = order[key]
            if label in record:
                if label == 'URL(s)':
                    for url in record['URL(s)']:
                        yield '\n <field><label>URL</label><content>%s</content></field>' %(makeUrlContent(url))
                elif label in ['Related Titles', 'Series', 'Source']:
                    yield '\n <field><label>%s</label><content>%s</content></field>' %(label, record[label])
                else:
                    yield '\n <field><label>%s</label><content>%s</content></field>' %(label, escape(record[label]))
        yield '\n</record>'

class RecordJSON(Observable):
    def __init__(self):
         Observable.__init__(self)

    def handleRequest(self, RequestURI='', **kwargs):
####### TP: toevoegen het afvangen van exceptions: 1. er is geen record id,
####### 2. er wordt geen metadata gevonden
        Scheme, Netloc, Path, Query, Fragment = urlsplit(RequestURI)
        arguments = parse_qs(Query)
        recordId = arguments.get('recordId', [''])[0]
        record = self.any.createRecord(recordId)
        yield okJSON()
        yield JSONEncoder().encode(record)
