import pprint
import gc

infra = None

from merescocore.components.statistics import Statistics, Logger

#from merescocore.components import StorageComponent, Lxml2Amara, XmlXPath, XmlPrintLxml, Amara2Lxml, Venturi, Configuration, FilterField, RenameField, TransformFieldValue, RewritePartname, Xml2Fields, XPath2Field, FilterMessages, Rss, RssItem, Fields2XmlTx, Reindex, ReindexConsole, XsltCrosswalk
from merescocore.components import StorageComponent, Lxml2Amara, XmlXPath, XmlPrintLxml, Amara2Lxml, Venturi, Configuration, FilterField, RenameField, TransformFieldValue, RewritePartname, Xml2Fields, XPath2Field, FilterMessages, RssItem, Fields2XmlTx, Reindex, ReindexConsole, XsltCrosswalk
from merescocore.components.statisticsxml import StatisticsXml
from merescocore.components.drilldown import SRUTermDrilldown, DrilldownFieldnames
from merescocore.components.http import PathFilter, FileServer, PathRename
#from merescocore.components.http.webrequestserver import WebRequestServer
from merescocore.components.sru import SruParser, SruHandler, SRURecordUpdate
from merescocore.components.tokenizefieldlet import TokenizeFieldlet

from merescocore.framework import Transparant, TransactionScope,  BatchTransactionScope, ResourceManager, Observable

from merescocomponents.facetindex import Drilldown, LuceneIndex, CQL2LuceneQuery, Fields2LuceneDocumentTx
from merescocomponents.facetindex.tools import unlock

# ImportError: No module named ngram
#from merescocomponents.ngram import NGramFieldlet, NGramQuery, RatioSuggester

from os.path import join, dirname, abspath, isdir
from os import remove, makedirs

# Anton: use homebrew dynamichtml from bzv instead of the cq2 version to support logging (fixme)
from bzv.dynamichtml import DynamicHtml

from delfttilburg.normalize import LastnameNormalize, FirstInitialNormalize
from bzv.yearnormalize import YearNormalize
#from delfttilburg.normalize import YearNormalize, LastnameNormalize, FirstInitialNormalize
from delfttilburg import RateRecordCookie, RateRecord, MultiLevelDrilldown, ComposeNameFromMods, ModsDateField, DidYouMean

## BZV specifics ###
from bzv.checkpart import CheckPart
from bzv.rss import Rss
from bzv.utils import HandleRequestPrinter, LimitQuery
from bzv.openurl import OpenURL
from bzv.composedaifrommods import ComposeDaiFromMods
from bzv.composefulljournaltitlefrommods import ComposeFullJournalTitleFromMods
from bzv.composejelfrommods import ComposeJELFromMods
from bzv.record import Record, RecordXml, RecordJSON
from bzv.googlelikequery import GoogleLikeQuery
from bzv.xmldocumentadapter import XmlDocumentAdapter
from bzv.writedeleted import WriteDeleted
from bzv.undelete import Undelete
from bzv.getavailability import GetAvailability

unqualifiedTermFields = [('mods', 1.0), ('fulltext.item.data', 0.5), ('meta', 0.1)]

drilldownFieldnames = [
#    'drilldown.mods.name.namePart',
#    'drilldown.mods.name.type',
#    'drilldown.mods.name.role.roleTerm',
    'drilldown.mods.classification',
    'drilldown.mods.genre',
    'drilldown.mods.language.languageTerm',
    'drilldown.mods.relatedItem.titleInfo.title',
    'drilldown.mods.subject.topic',
    'drilldown.meta.repository.id',
    'drilldown.meta.repository.repositoryGroupId',
    'drilldown.norm.date.unparsable',
    'drilldown.norm.date.year',
    'drilldown.norm.date.decennium',
    'drilldown.norm.date.centennium',
    'drilldown.norm.name.oneinitial',
    'drilldown.norm.name.lastname',
    'drilldown.norm.name.unparsable',
    'drilldown.norm.jel',
]

#we gebruiken als sort fields drilldown fields en meta fields
sortFieldnames = []
#    'sort.mods.originInfo.dateIssued',
#    'sort.norm.date.year',
#]

otherUntokenizedFieldnames = [
    'norm.dai',
    'norm.fulljournaltitle',
    'norm.docid'
]

metaFieldnames = [
    'meta.upload.id',
    'meta.record.id',
    'meta.repository.id',
    'meta.repository.set',
    'meta.repository.baseurl',
    'meta.repository.repositoryGroupId',
    'meta.repository.metadataPrefix',
    'meta.repository.collection',
    'meta.fulltext.url'
]

DRILLDOWN_PREFIX = 'drilldown.'
SORT_PREFIX = 'sort.'

namespaces = {
    'ddi'     :'http://www.icpsr.umich.edu/DDI',
    'didl'    :'urn:mpeg:mpeg21:2002:02-DIDL-NS',
    'dii'     :'urn:mpeg:mpeg21:2002:01-DII-NS',
    'document':'http://meresco.com/namespace/harvester/document',
    'fulltext':'http://meresco.com/namespace/fulltext',
    'oai'     :'http://www.openarchives.org/OAI/2.0/',
    'oai_dc'  :'http://www.openarchives.org/OAI/2.0/oai_dc/',
    'meta'    :'http://meresco.com/namespace/harvester/meta',
    'mods'    :'http://www.loc.gov/mods/v3',
    'rating'  :'http://meresco.com/namespace/record/rating'
}

class GarbageCollector(Observable):
    def handleRequest(self, Client=None, *args, **kwargs):
        gc.collect()
        return self.all.handleRequest(Client=Client, *args, **kwargs)

class ServerLog(Observable, Logger):
    def handleRequest(self, Client=None, *args, **kwargs):
        if Client:
            self.log(ip=Client[0])
        self.log(request='request')
        return self.all.handleRequest(Client=Client, *args, **kwargs)

class OurLogPrinter(Observable):
    def __init__(self, name):
        Observable.__init__(self)
        self._name = name

    def unknown(self, *args, **kwargs):
        print self._name, args, kwargs
        return self.all.unknown(*args, **kwargs)

class StatusReduce(Observable):
    def addField(self, name, value):
        if len(value) == 0:
            value = 'done'
        else:
            value = reduce(lambda a,b: a if a == 'done' else b, value)
        if value == '':
            value = 'failed'
        return self.do.addField(name=name, value=value)

def createUploadHelix(index, storageComponent, xsltFiles, reactor, infra):
    yearNormalize = YearNormalize(yearRange=(1000,2100))
    firstInitialNormalize = FirstInitialNormalize()
    lastnameNormalize = LastnameNormalize()

    serverConfig =  infra.configs['server']
    timeout = int(serverConfig.get('timeout',section='batchTransactions',default="1"))
    batchSize = int(serverConfig.get('batchSize',section='batchTransactions',default="50"))

    logger=infra.logger

    untokenized = drilldownFieldnames + sortFieldnames + metaFieldnames + otherUntokenizedFieldnames
    fields2LuceneDocument = \
        (ResourceManager("record",
                         lambda resourceManager:
                             Fields2LuceneDocumentTx(resourceManager,
                                                     untokenized=untokenized)),
         index
    )

    indexingHelix = \
        (Transparant(),
            fields2LuceneDocument,
            (FilterField(lambda name: DRILLDOWN_PREFIX + name in drilldownFieldnames),
                (RenameField(lambda name: DRILLDOWN_PREFIX + name),
                    fields2LuceneDocument
                )
            ),
            (FilterField(lambda name: SORT_PREFIX + name in sortFieldnames),
                (RenameField(lambda name: SORT_PREFIX + name),
                    fields2LuceneDocument
                )
            ),
        )

    normalizeTransactionStoreHelix = \
        (ResourceManager("record", lambda resourceManager:Fields2XmlTx(resourceManager, 'extra', namespace="http://meresco.com/namespace/fields/extra")),
            (storageComponent,)
        )

    storeModsHelix = \
        (RewritePartname('mods'),
            (XmlPrintLxml(),
                (storageComponent,)
            ),
        )

    indexModsHelix = \
        (XmlXPath(['//mods:mods'], namespaceMap=namespaces),
            (Xml2Fields(),
                indexingHelix,
                (RenameField(lambda name: name.split('.')[0]),
                    indexingHelix
                ),
                (FilterField(lambda name: name == '__id__'),
                    normalizeTransactionStoreHelix
                )
            ),
            (ComposeNameFromMods('norm.name.oneinitial'),
                (TransformFieldValue(firstInitialNormalize.normalize),
                    indexingHelix,
                    normalizeTransactionStoreHelix,
                ),
                (TransformFieldValue(firstInitialNormalize.unparsable),
                    (RenameField(lambda name: 'norm.name.unparsable'),
                        indexingHelix
                    )
                ),
                (TransformFieldValue(lastnameNormalize.normalize),
                    (RenameField(lambda name: 'norm.name.lastname'),
                        indexingHelix,
                        normalizeTransactionStoreHelix,
                    )
                )
            ),
            (ComposeDaiFromMods('norm.dai'),
                indexingHelix,
                normalizeTransactionStoreHelix,
            ),
            (ComposeFullJournalTitleFromMods('norm.fulljournaltitle'),
                indexingHelix,
                normalizeTransactionStoreHelix,
            ),
            (ComposeJELFromMods('norm.jel'),
                indexingHelix,
                normalizeTransactionStoreHelix,
            ),
#            (XPath2Field([('//mods:location/mods:url/text()', 'fulltext.url.exists')], namespaceMap=namespaces),
#                (TransformFieldValue(lambda value: 'true'),
#                    indexingHelix
#                )
#            ),
            (XPath2Field([('//mods:originInfo/mods:dateIssued/text()', 'norm.date.year')], namespaceMap=namespaces),
                (TransformFieldValue(yearNormalize.normalize),
                    indexingHelix,
                    normalizeTransactionStoreHelix,
                    (TransformFieldValue(lambda year: year[:len('YYY')]+'0'),
                        (RenameField(lambda name: 'norm.date.decennium'),
                            indexingHelix,
                            normalizeTransactionStoreHelix
                        )
                    ),
                    (TransformFieldValue(lambda year: year[:len('YY')]+'00'),
                        (RenameField(lambda name: 'norm.date.centennium'),
                            indexingHelix,
                            normalizeTransactionStoreHelix
                        )
                    )
                ),
                (TransformFieldValue(yearNormalize.unparsable),
                    (RenameField(lambda name: 'norm.date.unparsable'),
                        indexingHelix
                    )
                )
            ),
            (ModsDateField(),
                indexingHelix,
            ),
        )

    return \
        (BatchTransactionScope("batch", reactor=reactor,
                               batchSize=batchSize,
                               timeout=timeout),
            (TransactionScope("record"),
                (Venturi(
                        should=[
                             ('metadata', '/document:document/document:part[@name="metadata"]/text()'),
                             ('meta', '/document:document/document:part[@name="meta"]/text()')],
                        could=[
                            ('rating', '/rating:rating'),
                            ('header', '/document:document/document:part[@name="header"]/text()'),
                            ('fulltext', '/fulltext:fulltext'),
                            ('codeBook', '/ddi:codeBook')],
                        namespaceMap= namespaces
                    ),
                    (FilterMessages(allowed=['delete']),
                        index,
                        (WriteDeleted(),
                        (storageComponent,))
                    ),
                    (FilterMessages(allowed=['getStream']),
                        (storageComponent,)
                    ),

################### TP: storing of the required ('should') and optional ('could') parts
                    (XmlPrintLxml(),
                         (storageComponent,)
                    ),

                    (XmlXPath(['//mods:mods'], namespaceMap=namespaces),
                        indexModsHelix
                    ),

                    (XPath2Field([("//didl:Component/didl:Resource[@mimeType='application/pdf']/@ref", 'fulltext.url.exists')], namespaceMap=namespaces),
                        (TransformFieldValue(lambda value: 'true'),
                            indexingHelix
                        )
                    ),
                    (XPath2Field([("//oai:metadata/didl:DIDL/didl:Item/didl:Descriptor/didl:Statement/dii:Identifier/text()", 'norm.docid')], namespaceMap=namespaces),
                        indexingHelix,
                        normalizeTransactionStoreHelix,
                    ),
                    (XmlXPath(['/oai:metadata//record']),
                        (XsltCrosswalk(xsltFiles),
                            storeModsHelix,
                            indexModsHelix
                         )
                    ),

                    (XmlXPath(['/fulltext:fulltext'], namespaceMap=namespaces),
                        (RewritePartname('fulltext'),
                            (XmlPrintLxml(),
                                 (storageComponent,)
                            ),
                        ),
                        (Xml2Fields(),
                            indexingHelix
                        ),
                        (XPath2Field([('//fulltext:status/text()', 'fulltext.item.status')], namespaceMap=namespaces, sendAsList=True),
                            (StatusReduce(),
                                indexingHelix
                            ),
                        ),
                    ),

                    (XmlXPath(['/meta:meta']),
                        (Xml2Fields(),
                            indexingHelix,
                            (FilterField(lambda name: name == 'meta.repository.id'),
                                (RenameField(lambda name: 'meresco.exists'),
                                    (TransformFieldValue(lambda value: 'true'),
                                        indexingHelix
                                    )
                                )
                            ),
                            (RenameField(lambda name: 'meta'),
                                indexingHelix
                            )
                        ),
                    ),
                    (FilterMessages(disallowed=['delete']),
                        (Undelete(),
                        (storageComponent,))
                    )
                )
            )
        )

def main(reactor, observableHttpServer, sruUpdateServer, inbox, baseURL, portNumber, databasePath, configDict, infra, configWatcher):

    configs =  infra.configs

    logger = infra.logging.getLogger('meresco.server')
    websiteConfig = configs ['website']
    serverConfig  = configs ['server']

    applicationName = websiteConfig.get('applicationName',section='userFeedback', default='Beter Zoeken en Vinden')
    storageDirectory = join(databasePath, 'storage')
    indexDirectory = join(databasePath, 'index')

    dynamicPath = configDict['dynamicPath']
    staticPath  = configDict['staticPath']
    xsltPath    = configDict['xsltPath']
    templatePath =configDict['templatePath']

    # space seperated filelist
    picaplusXsltFileList = configDict['picaplusXsltFiles'].split()
    xsltFiles = [join(xsltPath, f) for f in picaplusXsltFileList ]

    unlock(indexDirectory)

    storageComponent = StorageComponent(storageDirectory, partsRemovedOnDelete=[])
    drilldownComponent = Drilldown(drilldownFieldnames, transactionName="batch")

    logger.info("Before LuceneIndex")
    indexHelix = \
            (LuceneIndex(indexDirectory, transactionName="batch"),
                (drilldownComponent,)
            )

    logger.info("After LuceneIndex")

    uploadHelix = createUploadHelix(indexHelix, storageComponent, xsltFiles, reactor=reactor,infra=infra)

    configurationComponent = Configuration(configDict)
    rateRecordList = RateRecordCookie(join(databasePath, 'ratedRecords'))

    statisticsPath = join(databasePath, 'statistics')
    isdir(statisticsPath) or makedirs(statisticsPath)
#FIXME statistics in 2.22 aangepast; even uitgezet
#    statistics = Statistics(statisticsPath, [('ip',), ('clause',)], 60)

    locator = serverConfig.get('Locator',section='proxies')
    if serverConfig.get('Testing', section='locator') == 'True':
        print "Testing locator"
        locator = "/availability" + serverConfig.get('Locator',section='proxies')
    else:
        print "Using acceptation/production locator"

    sru = \
        (SruParser(host=baseURL, port=portNumber, defaultRecordSchema='metadata', defaultRecordPacking='xml'),
            (SruHandler(),
                (CQL2LuceneQuery(unqualifiedTermFields),
                    indexHelix
                ),
                (storageComponent,),
                (SRUTermDrilldown(True),
                    (DrilldownFieldnames(
                        lambda field: DRILLDOWN_PREFIX + field,
                        lambda field: field[len(DRILLDOWN_PREFIX):]),
                            (drilldownComponent,)
                    ),
                    (CQL2LuceneQuery(unqualifiedTermFields),
                        indexHelix
                    ),
                )
            )
        )

    return \
        (Observable(),
            (configWatcher,
            ),
            (inbox,
                (XmlDocumentAdapter(),
                    uploadHelix,
                )
            ),
            (sruUpdateServer,
                (PathFilter('/update'),
                    (SRURecordUpdate(),
                        (Amara2Lxml(),
                            uploadHelix
                        )
                    )
                ),
                (PathFilter('/reindex'),
                    (ReindexConsole(),
                        (Reindex('meta'),
                            (FilterMessages(allowed=['listIdentifiers']),
                                (storageComponent,)
                            ),
                            (FilterMessages(disallowed=['listIdentifiers']),
                                uploadHelix
                             )
                        )
                    )
                )
            ),
            (observableHttpServer,
# FIXME statistics is veranderd in 2.22; even uitgezet
#                (statistics,
                    (ServerLog(), ),
#                    (PathFilter('/statistics'),
#                        (StatisticsXml(statistics), )
#                    ),
                    (PathFilter('/rss'),
                        (Rss(   title = applicationName,
                                description = 'Tilburg University',
                                link = baseURL,
                                maximumRecords = 15),
                            (CQL2LuceneQuery(unqualifiedTermFields),
                                indexHelix
                            ),
                            (CheckPart(parts=['mods'], action='stop'),
                                (storageComponent,),
                                (RssItem(
                                    nsMap=namespaces,
                                    title = ('metadata', '//mods:mods/mods:titleInfo/mods:title/text()'),
                                    description = ('metadata', '//mods:mods/mods:abstract/text()'),
                                    linkTemplate = baseURL + '/page/recordview?recordId=%(uploadId)s',
                                    uploadId = ('meta', '/meta:meta/meta:upload/meta:id/text()')),
                                    (storageComponent,)
                                ),
                                (CheckPart(parts=['mods'], action='continue'),
                                    (storageComponent,),
                                    (RssItem(
                                        nsMap=namespaces,
                                        title = ('mods', '//mods:mods/mods:titleInfo/mods:title/text()'),
                                        description = ('mods', '//mods:mods/mods:abstract/text()'),
                                        linkTemplate = baseURL + '/page/recordview?recordId=%(uploadId)s',
                                        uploadId = ('meta', '/meta:meta/meta:upload/meta:id/text()')),
                                       (storageComponent, )
                                    )
                                )
                            )
                        )
                    ),
                    (PathFilter('/record'),
                        (RecordXml(),
                            (Record(infra),
                                (storageComponent,)
                            )
                        )
                    ),
                    (PathFilter('/JSON'),
                        (RecordJSON(),
                            (Record(infra),
                                (storageComponent,)
                            )
                        )
                    ),
################### TP: this is only for testing
                    (PathFilter('/availability'),
                        (GetAvailability(configs),)
                    ),
                    (PathFilter('/eo/sru'),
                        (HandleRequestPrinter(),),
                        (LimitQuery(" AND (meta.repository.collection exact neeo OR meta.repository.collection exact repec)"),
                            sru
                        )
                    ),
                    (PathFilter('/sru'),
                        (HandleRequestPrinter(),),
                        sru
                    ),
                    (PathFilter('/files'),\
                        (PathRename(lambda path: path[len('/files'):]),
                            (FileServer(staticPath),)
                        )
                    ),
                    (PathFilter('/gc'),\
                        (GarbageCollector(),)
                    ),
                    (PathFilter('/', excluding=['/rss', '/record', '/JSON', '/availability', '/eo/sru', '/sru', '/update', '/files', '/statistics', '/reindex', '/gc']),
                        (DynamicHtml([dynamicPath], reactor=reactor, allowedModules=['cqlparser', 'cqlwebutils'],
                                     indexPage='/nl/page/search',
                                     infra = infra,
                                     configs = configs,
                                     templatePath = templatePath,
                                     ),
                            (GoogleLikeQuery(), ),
                            (CQL2LuceneQuery(unqualifiedTermFields),
                                indexHelix
                            ),
                            (OpenURL(locator),),
                            (Record(infra), (storageComponent,),),
                            (storageComponent,),
                            (drilldownComponent,),
                            (RateRecord(1,5),
                                (FilterMessages(allowed=['isAvailable', 'getStream']),
                                    (storageComponent,),
                                ),
                                uploadHelix
                            ),
                            (MultiLevelDrilldown({
                                    'date': [
                                        ('drilldown.norm.date.year', 20, False),
                                        ('drilldown.norm.date.decennium', 20, False),
                                        ('drilldown.norm.date.centennium', 20, False)
                                        ],
                                    'author': [
                                        ('drilldown.norm.name.oneinitial', 20, False),
                                        ('drilldown.norm.name.lastname', 10, True)
                                        ]
                                    }
                                ),
                                (drilldownComponent,),
                            ),
                            (configurationComponent,),
                            (rateRecordList, ),
                        )
                    )
#                )
            )
        )
