## begin license ##
#
#    "Delft-Tilburg" (delfttilburg) is a package containing shared code
#    between the Delft Discover and Tilburg "Beter Zoeken & Vinden" projects.
#    Both projects are based on Meresco Software (http://meresco.com)
#    Copyright (C) 2008 Technische Universiteit Delft http://www.tudelft.nl
#    Copyright (C) 2008 Universiteit van Tilburg http://www.uvt.nl
#    Copyright (C) 2008 Seek You Too (CQ2) http://www.cq2.nl
#
#    This file is part of "Delft-Tilburg".
#
#    "Delft-Tilburg" is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    "Delft-Tilburg" is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with "Delft-Tilburg"; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
## end license ##
from re import compile

class YearNormalize(object):
    def __init__(self, yearRange):
        self._yearBottom, self._yearTop = yearRange

        self._yearRes = [
             compile(r'(\d{2}XX)'), #20XX
             compile(r'(\d{4})'), #covers most cases
#            compile(r'^(\d{4})$'), #2008
#            compile(r'^(\d{4})\?$'), #2008?
#            compile(r'^(\d{4})-\d{2}-\d{2}$'), #2008-01-01
#            compile(r'^(\d{4})-\d{4}$'), #2008-2009
            compile(r'\[(\d{4})\]'), #[2008]
            compile(r'[a-z]+[.]? (\d{4})'), #cop. 2008; ca. 2008
            compile(r'\[[a-z]+[.]? (\d{4})\]'), #[cop. 2008]; [ca. 2008]
        ]

    def _normalize(self, aString, yearRe):
        try:
            match = yearRe.match(aString)
            if match:
                year = match.group(1).replace("XX", "00")
                if self._yearBottom <= int(year) <= self._yearTop:
                    return year
        except TypeError:
            return None

    def normalize(self, aString):
        for yearRe in self._yearRes:
            result = self._normalize(aString, yearRe)
            if result != None:
                return result
        return None

    def unparsable(self, aString):
        return self.normalize(aString) == None and aString or None
