import os
os.environ['LANG']='en_US.UTF-8'
import unittest
from bzv.customerConfig import CustomerConfig, CustomerConfigException
from bzv.infra import Infra
infra = None

class CustomerConfigTest(unittest.TestCase):
    # N.B. Tests are run in alphabetical order.
    #
    def makeInfra(self):
        global infra
        loggingConfigfile = 'logging.conf'
        infra = Infra('Running customerConfigTest', loggingConfigfile=loggingConfigfile)
        infra.logger.debug("Infra logs successfully")

    def setUp(self):
        global infra
        if (infra == None):
            self.makeInfra()
        configfile = 'data/customerConfig.cfg'
        self.config = CustomerConfig(configfile,infra)

    def testInfra(self):
        infra.logger.debug("Test Infra: OK" )
        self.failUnless(infra.status == True )

    def testGetExistingOption(self):
        foo = self.config.get(option = 'foo', section='testSection')
        self.failUnless(foo == 'bar' )

    def testGetNoneExistingOptionWithDefault(self):
        foo = self.config.get(option = 'noneexistent', section = 'testSection',default = 'defaultvalue')
        self.failUnless(foo == 'defaultvalue' )

    def testGetNoneExistingOptionWithoutDefault(self):
        foo = self.config.get(option = 'noneexistent', section ='testSection')
        self.failUnless(foo == 'noneexistent' )

    def testGetNoneExistingSection(self):
        foo = self.config.get(option = 'defaultvalue', section='nonexistingSection')
        self.failUnless(foo == 'defaultvalue' )

    def testGetEmptyOption(self):
        self.failUnlessRaises(CustomerConfigException, self.config.get, option = None, section = 'testSection')

    def testGetOptionWithEmptySection(self):
        self.failUnlessRaises(CustomerConfigException, self.config.get, option = 'foo', section = None)

    def testIncorrectConfigfilename(self):
        configfile = 'data/NONE-EXISTENT.cfg'
        config = CustomerConfig(configfile,infra)
        foo = config.get(option = 'foo', section='testSection')
        self.failUnless(foo == 'foo')

if __name__ == '__main__':
    unittest.main()
