#!/usr/bin/env python
from __future__ import with_statement

from glob import glob
from sys import path
for directory in glob('../deps.d/*'):
    path.insert(0, directory)
path.insert(0, '..')

from weightless import Reactor
from sys import stdout
from os.path import abspath, dirname, join, isdir, basename
from os import makedirs
from merescocore.components.http import ObservableHttpServer
from merescocore.components.http.webrequestserver import WebRequestServer
from merescocore.components.sru import SRURecordUpdate
from merescocore.framework import Observable, be
from re import compile
from traceback import format_exc
from amara.binderytools import bind_stream

portNumber = 8888
mydir = dirname(abspath(__file__))
dumpdir = join(mydir, 'sahara_output')
notWordCharRE = compile('\W+')

class Dump(SRURecordUpdate):
    def __init__(self, dumpdir=dumpdir):
        SRURecordUpdate.__init__(self)
        self._dumpdir = dumpdir
        self._number = self._findLastNumber()

    def handleRequest(self, httpRequest):
        try:
            updateRequest = bind_stream(httpRequest.content).updateRequest
            recordId = str(updateRequest.recordIdentifier)
            normalizedRecordId = notWordCharRE.sub('_', recordId)
            self._number +=1
            filename = '%05d_%s.updateRequest' %(self._number, normalizedRecordId)
            with open(join(self._dumpdir, filename), 'w') as f:
                print recordId
                stdout.flush()
                updateRequest.xml(f)
            self.writeSucces(httpRequest)
        except Exception, e:
            self.writeError(httpRequest, format_exc(limit=7))

    def _findLastNumber(self):
        return max([int(basename(f)[:5]) for f in glob(join(self._dumpdir, '*.updateRequest'))]+[0])


def main(reactor):
    isdir(dumpdir) or makedirs(dumpdir)
    server = be(
        (Observable(),
            (ObservableHttpServer(reactor, portNumber),
                (WebRequestServer(),
                    (Dump(dumpdir),)
                )
            )
        )
    )
    server.once.observer_init()

if __name__== '__main__':
    reactor = Reactor()
    main(reactor)
    print 'Ready to rumble the dumpserver at', portNumber
    print '  - dumps are written to', dumpdir
    stdout.flush()
    reactor.loop()