
from unittest import TestCase
from bzv.editorialdate2dateissued import createDateIssued, parseDateRange

class EditorialDate2DateIssuedTest(TestCase):

    def testDateSplit(self):
        self.assertDate(('2008-09',), '200809')
        self.assertDate(('2008-09-01',), '20080901')
        self.assertDate(('2008',), '2008')
        self.assertDate(('2008-04','2008-05'), '200804/05')
        self.assertDate(('2008-05-17', '2008-05-23'), '20080517/23')
        self.assertDate(('2008-05-28', '2008-06-03'), '20080528/0603')
        self.assertDate(('2008', '2009'), '2008/2009')
        self.assertDate(('2002-05', '2005-01'), '200205/200501')
        self.assertDate(('2001-03',), '200121')
        self.assertDate(('2001-06',), '200122')
        self.assertDate(('2001-09',), '200123')
        self.assertDate(('2001-12',), '200124')
        self.assertDate(('2001-01',), '200131')
        self.assertDate(('2001-04',), '200132')
        self.assertDate(('2001-07',), '200133')
        self.assertDate(('2001-10',), '200134')
        self.assertDate(('2005-05',), '2005/05')
        self.assertDate(('2005-05', '2005-05-06'), '200505/0506')
        self.assertDate(('2005-05', '2005-05-06'), '200505/0506')
        self.assertDate(('2007-12-22', '2008-01-04'), '20071222/20080104')
        
    def testDateIssued(self):
        self.assertEquals('<dateIssued qualifier="inferred" encoding="iso8601">2008-09</dateIssued>', createDateIssued('200809'))
        self.assertEquals('<dateIssued qualifier="inferred" encoding="iso8601" point="start">2008-09</dateIssued><dateIssued qualifier="inferred" encoding="iso8601" point="end">2008-10</dateIssued>', createDateIssued('200809/10'))

    def testGatheredLiveData(self):
        dates = (date.strip() for date in open('data/_dt.fields') if date.strip())
        for date in dates:
            self.assertTrue(parseDateRange(date) != None, date)

    def testDateIssuedWithoutValidDataRaisesError(self):
        try:
            createDateIssued('98765432310')
            self.fail()
        except ValueError, e:
            self.assertTrue('98765432310' in str(e))

    def assertDate(self, expected, date):
        self.assertEquals(expected, parseDateRange(date))