from unittest import TestCase
from StringIO import StringIO
from weightless import compose
from cq2utils import autostart


from bzv.elsevierreader import readToc, readEditorialItem, nextValue, nextCode

class ElsevierReaderTest(TestCase):

    def testReadExample(self):
        r = []
        voer = compose(readToc(collector(r)))
        toc = StringIO(EXAMPLE)
        voer.next()
        for line in toc:
            voer.send(line)
        voer.throw(StopIteration())
        voer.close()

        self.assertEquals(3, len(r))
        editorialItem = r[0]
        self.assertEquals(12, len(editorialItem), editorialItem.keys())

        self.assertEquals(['Morello, J.A.'], editorialItem['_au'])
        self.assertEquals(2, len(editorialItem['_ii']))
        self.assertEquals(['Josephine A. Morello. Department of Pathology, The University of Chicago, Chicago, Illinois'], editorialItem['_ca'])
        t2 = editorialItem['parent']
        self.assertEquals('CXT0218A 01964399 00300001', t2['_t2'][0])
        t1 = t2['parent']
        self.assertEquals('CXT0218A 01964399', t1['_t1'][0])
        t0 = t1['parent']
        self.assertEquals('CXT0218A', t0['_t0'][0], t2)

    def testReadEditorialITem(self):
        r = []
        voer = compose(readEditorialItem({'_t2':['ape']}, collector(r)))
        toc = StringIO("""t3Identifier
_ii aap noot mies
_mf laatste""")
        voer.next()
        for line in toc:
            voer.send(line)
        voer.throw(StopIteration())
        voer.close()
        self.assertEquals(1, len(r))
        self.assertEquals({'_t3':['t3Identifier'], '_ii':['aap noot mies'], '_mf':['laatste'], 'parent':{'_t2':['ape']}}, r[0])

    def testNextValue(self):
        result = []
        def codeReader(r):
            for n in range(10):
                code = yield nextCode()
                value = yield nextValue()
                r.append((code, value))
        voer = compose(codeReader(result))
        voer.next()
        voer.send('key line1')
        voer.send('    line2')
        voer.send('    line.last')
        voer.send('ke2 value2')
        voer.send('    line2.last')
        voer.throw(StopIteration())
        self.assertEquals([('key', 'line1 line2 line.last'), ('ke2', 'value2 line2.last')], result)


    def testReadDatasetFile(self):
        r = []
        voer = compose(readToc(collector(r)))
        voer.next()
        for line in open('data/voorbeeld-dataset.toc'):
            voer.send(line)
        voer.throw(StopIteration())
        voer.close()
        self.assertEquals(330, len(r))
        self.assertEquals('CXT0218A 01761617 01650001 07001381', r[-1]['_t3'][0])





@autostart
def collector(result):
    while True:
        result.append((yield))

EXAMPLE = """_t0 CXT0218A
_vn 4.0
_pd 20071226
_t1 CXT0218A 01964399
_jn Clinical Microbiology Newsletter
_cr Copyright (c) 2007 Elsevier Inc.
_t2 CXT0218A 01964399 00300001
_vl 30
_is 1
_pr 1-8
_dt 20080101
_t3 CXT0218A 01964399 00300001 07000554
_ii S0196-4399(07)00055-4
_ii [DOI] 10.1016/j.clinmicnews.2007.12.001
_ty FLA
_li EN
_ti Clinical Microbiology Newsletter at Age 30 - History and Evolution
_au Morello, J.A.
_ca Josephine A. Morello. Department of Pathology, The University of
    Chicago, Chicago, Illinois
_ab In 1979, the first issue of a new newsletter for clinical
    microbiologists was published. The purpose of Clinical Microbiology
    Newsletter (CMN), a twice-a-month publication, was to provide clinical
    microbiology laboratorians, clinical pathologists, and infectious disease
    clinicians with concise mini-reviews of recent findings in the field.
    Editorial opinions, case reports of unusual organisms or infections, and
    several other features were included. The success of the Newsletter was
    immediate. Although the format and editorship have undergone changes, CMN
    is now beginning its 30th year of publication and is available online, as
    well as in print format. CMN continues its mission of providing readers
    with the most up-to-date information of value in their laboratory
    endeavors.
_la EN
_pg 1-5
_mf [XML Cit] 07000554
_t3 CXT0218A 01964399 00300001 07000566
_ii S0196-4399(07)00056-6
_ii [DOI] 10.1016/j.clinmicnews.2007.12.002
_ty FLA
_li EN
_ti Successful Use of Linezolid in the Treatment of Enterococcal
    Endocarditis: an Effective and Safe Option?
_au Javaid, M.
_au Abdelhamed, A.
_au Samar, H.
_au Cynamon, M.
_ca Michael Cynamon. Veterans Affairs Medical Center, Syracuse, New York
_pg 5-6
_mf [XML Cit] 07000566
_t3 CXT0218A 01964399 00300001 07000578
_ii S0196-4399(07)00057-8
_ii [DOI] 10.1016/j.clinmicnews.2007.12.003
_ty ANN
_li EN
_ti New CLSI Documents
_pg 7
_mf [XML Cit] 07000578
"""