from cq2utils import CQ2TestCase, CallTrace
from bzv.fulltextextractor.fulltextextractor import FullTextExtractor
from os.path import join, dirname, abspath

class FullTextTest(CQ2TestCase):

    def testFailedUrlOpen(self):
        def failedUrlopen(url):
            raise Exception("cannot open <iostream.h>")
        observer = CallTrace()
        observer.urlopen = failedUrlopen
        ft = FullTextExtractor()
        ft.addObserver(observer)
        ft._datestamp = lambda: "DATESTAMP"
        ft.addFullText(identifier='id0', urls=['http://example.org/data.pdf?a=b&c=d', 'otherurl'])
        self.assertEquals(1, len(observer.calledMethods))
        self.assertEqualsWS("""
        <fulltext xmlns="http://meresco.com/namespace/fulltext">
            <identifier>id0</identifier>
            <datestamp>DATESTAMP</datestamp>
            <item>
                <url>http://example.org/data.pdf?a=b&amp;c=d</url>
                <status>failed</status>
                <errorCode>cannot open &lt;iostream.h&gt;</errorCode>
                <data></data>
            </item>
            <item>
                <url>otherurl</url>
                <status>failed</status>
                <errorCode>cannot open &lt;iostream.h&gt;</errorCode>
                <data></data>
            </item>
        </fulltext>
        """, observer.calledMethods[0].kwargs['recordData'])

    def testReadPlainTest(self):
        def urlOpenWithResult(self):
            for line in ['this ', 'is ', 'some ', 'full', 'text']:
                yield line
        observer = CallTrace()
        observer.urlopen = urlOpenWithResult
        ft = FullTextExtractor()
        ft.addObserver(observer)
        ft._pdfToText = lambda pdfData: "converted"
        ft._isPdf = lambda filename: True
        ft._datestamp = lambda: "DATESTAMP"
        ft.addFullText(identifier='id1', urls=["http://exists.nl"])

        self.assertEquals(1, len(observer.calledMethods))
        self.assertEqualsWS("""
        <fulltext xmlns="http://meresco.com/namespace/fulltext">
            <identifier>id1</identifier>
            <datestamp>DATESTAMP</datestamp>
            <item>
                <url>http://exists.nl</url>
                <status>done</status>
                <errorCode></errorCode>
                <data>converted</data>
            </item>
        </fulltext>
        """, observer.calledMethods[0].kwargs['recordData'])

    def testPdfToText(self):
        ft = FullTextExtractor()
        result = ft._pdfToText(join(dirname(abspath(__file__)), 'test.pdf'))
        self.assertEquals("A small example pdf for testing", result)

    def testIsPdf(self):
        ft = FullTextExtractor()
        self.assertFalse(ft._isPdf('/bin/ls'))
        self.assertTrue(ft._isPdf(join(dirname(abspath(__file__)), 'test.pdf')))

