#!/usr/bin/env python

from __future__ import with_statement

from os import system, makedirs
system('find .. -name "*.pyc" | xargs rm -f')

from glob import glob
import sys
for depspath in glob('../deps.d/*'):
    sys.path.insert(0, depspath)
sys.path.insert(0, '..')

from unittest import TestCase, main
from cq2utils import CallTrace, autostart, getRequest, postRequest, wheelOfTime
from os.path import join, isdir
from time import sleep, time
from amara.binderytools import bind_file
from merescocore.components.http import ObservableHttpServer
from merescocore.framework import be

from bzv.oaiserver import main as serverMain
from weightless import Reactor, compose
from socket import socket
from random import randint
from StringIO import StringIO

global port
global reactor
port = None
reactor = None
integrationTempdir = './integrationtest'
inboxDirectory = join(integrationTempdir, 'inboxDirectory')
doneDirectory = join(integrationTempdir, 'doneDirectory')

class OAIIntegrationTest(TestCase):


    def testOaiError(self):
        header, response = getRequest(reactor, port, '/elsevier/oai', {'verb':'NoVerb'})
        self.assertTrue('badVerb' in response.xml(), response.xml())

    def testOaiIdentify(self):
        # test correct XML
        header, response = getRequest(reactor, port, '/elsevier/oai', {'verb':'Identify'})
        self.assertTrue('OAI' in response.xml(), response.xml())

    def testOaiMods(self):
        header, response = getRequest(reactor, port, '/elsevier/oai', {'verb':'ListRecords', 'metadataPrefix':'mods'})
        self.assertEquals(24, len(response.OAI_PMH.ListRecords.record))

    def testOaiDc(self):
        header, response = getRequest(reactor, port, '/elsevier/oai', {'verb':'ListRecords', 'metadataPrefix':'oai_dc'})
        self.assertFalse('<error code="' in response.xml(), response.xml())
        self.assertEquals(24, len(response.OAI_PMH.ListRecords.record))

    def testOaiDcGetRecord(self):
        header, response = getRequest(reactor, port, '/elsevier/oai', {'verb':'GetRecord', 'metadataPrefix':'oai_dc', 'identifier':'elsevier:CXT0242A:01988859:00690004:08000396'})
        self.assertEquals(1, len(response.OAI_PMH.GetRecord.record))

    def testOaiListSets(self):
        header, response = getRequest(reactor, port, '/elsevier/oai' , {'verb':'ListSets'})
        self.assertTrue('<setSpec>elsevier</setSpec>' in response.xml(), response.xml())

    def testOaiListMetadataFormats(self):
        header, response = getRequest(reactor, port, '/elsevier/oai' , {'verb':'ListMetadataFormats'})
        self.assertTrue('<metadataPrefix>mods</metadataPrefix>' in response.xml(), response.xml())

def createDatabase():
    start = time()
    print "Creating database in", integrationTempdir
    system('cp -r data/CXT0242A %(integrationTempdir)s' % globals())
    system('mv %(integrationTempdir)s/CXT0242A %(inboxDirectory)s/any_name' % globals())

    print doneDirectory
    while not glob(join(doneDirectory, 'any_name*')):
        reactor.addTimer(0.01, lambda: None)
        reactor.step()

    print "Finished creating database in %s seconds" % (time() - start)
    #print "Giving the server index some time for a auto refresh"
    #wheelOfTime(reactor, 3)
    print "Finished creating database"


if __name__ == '__main__':
    from sys import argv
    if not '--fast' in argv:
        system('rm -rf ' + integrationTempdir)
        makedirs(inboxDirectory)
        makedirs(doneDirectory)
    reactor = Reactor()

    def createServer():
        global port
        port = randint(50000,60000)
        observableHttpServer = ObservableHttpServer(reactor, port)

        processorHelix, serverHelix = serverMain(reactor, observableHttpServer,
            inboxDirectory=inboxDirectory,
            doneDirectory=doneDirectory,
            databasePath=integrationTempdir,
            oaiAdminEmail='info@example.org',
            oaiRepositoryName='IntegrationTest.',
            stdoutStream=StringIO(),
            stderrStream=StringIO())

        processor, server = be(processorHelix), be(serverHelix)
        #processor.printTree()
        #server.printTree()
        processor.once.observer_init()
        server.once.observer_init()
    createServer()

    if '--fast' in argv and isdir(integrationTempdir):
        argv.remove('--fast')
        print "Reusing database in", integrationTempdir
    else:
        createDatabase()
        print 'Recreate server to test for persistency'
        createServer()
    print port
    main()



