
from unittest import TestCase
from cgi import parse_qs
from bzv.openurl import OpenURL
from urlparse import urlsplit

from lxml.etree import parse

class OpenURLTest(TestCase):
    def testOne(self):
        o = OpenURL('http://radix-21.uvt.nl:4080/locator/request')
        lxmlNode = parse(open('data/elsevier:CXT0242A:00092614:04570004:08004910.mods.xml'))
        scheme, netloc, path, query, fragment = urlsplit(o.createOpenURL(lxmlNode))
        result = parse_qs(query)

        self.assertEquals(['article'], result['genre'])
        self.assertEquals(['Feng'], result['aulast'])
        self.assertEquals(['Wenhui'], result['aufirst'])
        #auinit ????
        self.assertEquals(['Theoretical study of the O(3P)+HCNO reaction'], result['atitle'])
        self.assertEquals(['00092614'], result['issn'])
        self.assertEquals(['Chemical Physics Letters'], result['title'])
        self.assertEquals(['457'], result['volume'])
        self.assertEquals(['4'], result['issue'])
        self.assertEquals(['307'], result['spage'])
        self.assertEquals(['311'], result['epage'])
        self.assertEquals(['307-311'], result['pages'])
        #self.assertEquals(['2008-04-04'], result['date'])
        self.assertEquals(['2008'], result['date'])


    def testOther(self):

        o = OpenURL('url')
        lxmlNode = parse(open('data/neeo:oai:wo.uvt.nl:3125798.mods.xml'))
        result = parse_qs(o._create(lxmlNode))
        self.assertEquals(['bookitem'], result['genre'])
        self.assertEquals(['Bovenberg'], result['aulast'])
####### last '.' is stripped; that is not what you want in this case?
        self.assertEquals(['A.L'], result['aufirst'])
        self.assertEquals(['9781847206602'], result['isbn'])
        self.assertEquals(['Frontiers in pension finance and reform: Institutional innovation in the Netherlands'], result['atitle'])
        self.assertEquals(['Frontiers in Pension Finance'], result['title'])


    def testReverseEngineerThomasExamples(self):
        reversed = parse_qs('genre=article&issn=00220655&volume=45&issue=1&oissue=1&spage=99&pages=99-117&aulast=Ark&aufirst=L&auinit1=L&atitle=Detecting+answer+copying+using+alternate+test+forms+and+seat+locations+in+small-scale+examinations&title=Journal+of+Educational+Measurement&date=2008')
        # This example is a OpenURL which uses a Object-Metadata Zone as the Object-Description
        objectMetadataZone_keys = set(['genre', 'aulast', 'aufirst', 'auinit', 'auinit1', 'auinitm', 'coden', 'issn', 'eissn', 'isbn', 'title', 'stitle', 'atitle', 'volume', 'part', 'issue', 'spage', 'epage', 'pages', 'artnum', 'sici', 'bici', 'ssn', 'quarter', 'date'])
        notInObjectMetadataZone_specs = set(['oissue'])
        extended = objectMetadataZone_keys.union( notInObjectMetadataZone_specs)
        for key in reversed.keys():
            self.assertTrue(key in extended,  key)
