
from merescocore.components import Crosswalk

from unittest import TestCase
from lxml.etree import parse, tostring, XMLParser
from difflib import unified_diff
from os.path import join, dirname, abspath
from StringIO import StringIO

from merescocore.components.xml_generic import Validate, ValidateException
from merescocore.components import XsltCrosswalk
from merescocore.framework import Observable, be
from cq2utils import CallTrace
from merescocore.components import XmlPrintLxml

picaplus2modsXsltFiles = ['../xslt/picaplus2mods.xsl']
modsxsd = '../schemas/mods-3-3.xsd'

class PicaPlus2ModsTest(TestCase):
    """
    Example records retrieved from
    http://radix-21.uvt.nl:4080/ncc/pica+?number=3&query=rest

    (Numbers 3,4,5,6)
    """

    def testPicaRecords(self):
        self.assertRecord('pica_record_003')
        self.assertRecord('pica_record_004')
        self.assertRecord('pica_record_005')
        self.assertRecord('pica_record_006')

    def openAndPatch(self, id):
        """ we wrap the example records in collection tags to make the xslt work."""
        xsltText = open('xsltdata/%s.xml' % id).read()

        return parse(StringIO(xsltText))

    def assertRecord(self, id):
        self.crosswalkedNode = []
        inputRecord = self.openAndPatch(id)
        expectedMODSResult = tostring(parse(open('xsltdata/%s.MODS.xml' % id), XMLParser(remove_blank_text=True)), pretty_print = True)

        class Intercept:
            def someMessage(innerself, xmlNode):
                self.crosswalkedNode.append(tostring(xmlNode, pretty_print=True))
        start = be (
            (Observable(),
                (XsltCrosswalk(picaplus2modsXsltFiles),
                    (Intercept(), ),
                    (Validate(modsxsd), )
                )
            )
        )

        try:
            start.do.someMessage(inputRecord)
        except ValidateException, e:
            if self.crosswalkedNode:
                for nr, line in enumerate(self.crosswalkedNode[-1].split('\n')):
                    print nr+1, line
            raise

        diffsMODS = list(unified_diff(self.crosswalkedNode[0].split('\n'), expectedMODSResult.split('\n'), fromfile='result', tofile='expected', lineterm=''))

        self.assertFalse(diffsMODS, '\n' + '\n'.join(diffsMODS))
