from cq2utils import CQ2TestCase, CallTrace
from bzv.fulltextextractor.sruupdateclient import SruUpdateClient, SruUpdateClientException
from StringIO import StringIO

class SruUpdateClientTest(CQ2TestCase):

    def testSuccess(self):
        requests = []
        def urlopen(*args):
            requests.append(args)
            return StringIO("""<?xml version="1.0" encoding="UTF-8"?>
<srw:updateResponse xmlns:srw="http://www.loc.gov/zing/srw/" xmlns:ucp="info:lc/xmlns/update-v1">
    <srw:version>1.0</srw:version>
    <ucp:operationStatus>success</ucp:operationStatus>
</srw:updateResponse>""")

        client = SruUpdateClient('localhost', 8888, '/sru')
        client._urlopen = urlopen
        client.sendReplace('recordId', recordPacking="xml", recordSchema="fulltext", recordData="<data>this is fulltext</data>")
        self.assertEquals(1, len(requests))

        url, data = requests[0]

        self.assertEquals("http://localhost:8888/sru", url)
        self.assertEqualsWS('<?xml version="1.0" encoding="UTF-8"?><updateRequest xmlns:srw="http://www.loc.gov/zing/srw/" xmlns:ucp="http://www.loc.gov/ucp"><srw:version>1.0</srw:version><ucp:action>info:srw/action/1/replace</ucp:action><ucp:recordIdentifier>recordId</ucp:recordIdentifier><srw:record><srw:recordPacking>xml</srw:recordPacking><srw:recordSchema>fulltext</srw:recordSchema><srw:recordData><data>this is fulltext</data></srw:recordData></srw:record></updateRequest>', data)


    def testFail(self):
        requests = []
        def urlopen(*args):
            requests.append(args)
            return StringIO("""<?xml version="1.0" encoding="UTF-8"?>
<srw:updateResponse xmlns:srw="http://www.loc.gov/zing/srw/" xmlns:ucp="info:lc/xmlns/update-v1">
    <srw:version>1.0</srw:version>
    <ucp:operationStatus>fail</ucp:operationStatus>
    <srw:diagnostics>
    <diag:diagnostic xmlns:diag="http://www.loc.gov/zing/srw/diagnostic/">
        <diag:uri>info:srw/diagnostic/12/1</diag:uri>
        <diag:details>ERROR_BY_TESTING</diag:details>
        <diag:message>Invalid component:  record rejected</diag:message>
    </diag:diagnostic>
</srw:diagnostics>
</srw:updateResponse>""")

        client = SruUpdateClient('localhost', 8888, '/sru')
        client._urlopen = urlopen

        try:
            client.sendReplace('recordId', recordPacking="xml", recordSchema="fulltext", recordData="<data>this is fulltext</data>")
            self.fail("Should not reach here")
        except SruUpdateClientException, e:

            self.assertTrue("ERROR_BY_TESTING" in str(e), str(e))
