from cq2utils import CQ2TestCase, CallTrace

from os.path import join
from lxml.etree import parse
from StringIO import StringIO

from bzv import XmlDocumentAdapter

class XmlDocumentAdapterTest(CQ2TestCase):
    def setUp(self):
        CQ2TestCase.setUp(self)
        self.adapter = XmlDocumentAdapter()
        self.observer = CallTrace('observer')
        self.adapter.addObserver(self.observer)

    def testValidFile(self):

        self.assertEquals(('', ''), self.adapter._splitFilename("some_file"))
        self.assertEquals(('', ''), self.adapter._splitFilename("repository:identifier"))
####### add col:group:
        self.assertEquals(('col', 'group', 'repository', 'identifier'), self.adapter._splitFilename("col:group:repository:identifier.record"))
####### add col:group:
        self.assertEquals(('col', 'group', 'repo', 'Even\tTabs and <spaces> are /escaped properly'), self.adapter._splitFilename("col:group:repo:Even%09Tabs and <spaces> are %2Fescaped properly.record"))

    def testCreateXml(self):
#        filename = 'repository:some:identifier:1.record'
        filename = 'col:group:repository:some:identifier:1.record'

####### problem with prefix/namespace; it should be <oai:record/>
        self.adapter.add(identifier=filename, lxmlNode=parse(StringIO('<record/>')))

        self.assertEquals("add(identifier='repository:some:identifier:1', lxmlNode=<etree._ElementTree>)", str(self.observer.calledMethods[0]))

        lxmlNode = self.observer.calledMethods[0].kwargs['lxmlNode']
####### 2 parts instead of 3: meta and metadata; header is no longer created
        self.assertEquals(2, len(lxmlNode.xpath('/document:document/document:part', namespaces={'document':'http://meresco.com/namespace/harvester/document'})))

####### header is no longer relevant
#        header = lxmlNode.xpath('/document:document/document:part[@name="header"]/text()', namespaces={'document':'http://meresco.com/namespace/harvester/document'})[0]
#        self.assertEqualsWS("""<header xmlns="http://www.openarchives.org/OAI/2.0/">
#            <identifier>some:identifier:1</identifier>
#        </header>""", header)

        meta = lxmlNode.xpath('/document:document/document:part[@name="meta"]/text()', namespaces={'document':'http://meresco.com/namespace/harvester/document'})[0]
        self.assertEqualsWS("""<meta:meta xmlns:meta="http://meresco.com/namespace/harvester/meta">
  <meta:upload><meta:id>repository:some:identifier:1</meta:id></meta:upload>
  <meta:record>
    <meta:id>some:identifier:1</meta:id>
  </meta:record>
  <meta:repository>
    <meta:id>repository</meta:id>
    <meta:repositoryGroupId>group</meta:repositoryGroupId>
    <meta:collection>col</meta:collection>
  </meta:repository>
</meta:meta>""", meta)

        metadata = lxmlNode.xpath('/document:document/document:part[@name="metadata"]/text()', namespaces={'document':'http://meresco.com/namespace/harvester/document'})[0]
####### problem with prefix/namespace; it should be <oai:recor/>
        self.assertEqualsWS("""<oai:metadata xmlns:oai="http://www.openarchives.org/OAI/2.0/"><record/></oai:metadata>""", metadata)
