<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet
	xmlns="http://www.loc.gov/mods/v3"
	xmlns:a="http://drcwww.uvt.nl/~place/search%20engine/extension"
	xmlns:srw="http://www.loc.gov/zing/srw/"
	xmlns:diag="http://www.loc.gov/zing/srw/diagnostic/"
 	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	exclude-result-prefixes="srw diag"
	version="1.0">
	<xsl:output encoding="UTF-8" indent="yes" method="xml"/>

	<xsl:template match="/srw:searchRetrieveResponse|/collection">
		<modsCollection
			xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
			xsi:schemaLocation="http://www.loc.gov/mods/v3 http://www.loc.gov/mods/v3/mods-3-3.xsd">
			<xsl:apply-templates select="/srw:searchRetrieveResponse/srw:records/srw:record/srw:recordData/record|collection/record"/>
		</modsCollection>
	</xsl:template>

	<xsl:template match="/record|record">
		<mods
			xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
			xsi:schemaLocation="http://www.loc.gov/mods/v3 http://www.loc.gov/mods/v3/mods-3-3.xsd">
			<xsl:variable name="type">
				<xsl:value-of select="substring(datafield[@tag='002@']/subfield[@code='0'],1,2)"/>
				<!--	fysieke verschijningsvorm
					A_ print
					B_ audiovisueel materiaal
					C_ materiaal voor blinden
					E_ microformaat
					G_ geluidsdrager
					H_ handschrift
					K_ kaart
					M_ bladmuziek
					O_ online resource
					S_ elektronische informatiedrager
					V_ fysiek object
					Z_ combi van materiaalsoorten
					bibliografische verschijningsvorm
					_a monografie
					_b tijdschift/periodiek
					_c meerdelig werk
					_f onzelfstandig deel van een meerdelige publicatie
					_g
					_m
					_l
					_s
					_z onbekend
				-->
				<!--	Uit http://www.oclcpica.org/?id=13&ln=nl&par=p-103:
					positie 1
					A = Boekmateriaal, Onderdeelbeschrijvingen, Online Contents en Oude Drukken
					B = AV materiaal
					C = (ooit: materiaal gedrukt voor 1800)
					D = brieven
					F = handschriften
					G = geluidsdragers
					I = illustratief materiaal
					K = Kaartmateriaal
					M = Gedrukte muziek
					O = online resources
					S = software gegevens
					V = objecten

					positie 2
					a = zelfstandige uitgave, monografie (de beschrijving heeft geen relatie met andere beschrijvingen) / deeltitel reeks
					b = seriele publicatie / koepeltitel seriele publicatie (=reeks)
					c = koepeltitel (MP)
					d = koepeltitel reeks
					e = onzelfstandige tussentitel (MP)
					E = zelfstandige tussentitel (MP)
					f = onzelfstandige deeltitel (MP)
					F = zelfstandige deeltitel (MP)
					g = deelbeschrijving reeks
					h = b+c+d (BNB-tape-titels)
					j = c+d+b+f+g+k (BNB-tape-titels)
					k = b+d (BNB-tape-titels)
					l = koepeltitel MP in reeks
					m = onzelfstandig deel MP in reeks
					M = zelfstandig deel MP in reeks
					o = online contents
					r = convoluut of compilatie
					s = onderdeelbeschrijving, artikelbeschrijving
				-->
				<!-- Cherry in een mailtje aan Ko:
				> Grofweg gezegd kan je het volgende stellen (ik weet dat mijn opmerking je
				> niet veel zal helpen, maar toch):
				> - een eBook dat we "apart" hebben gecatalogiseerd heeft de aanduiding: Oax
				> - een eBook dat we "apart" hebben gecatalogiseerd en dat onderdeel is van
				> een reeks heeft de aanduiding: Ogx
				> - een e-tijdschrift dat we "apart" hebben gecatalogiseerd heeft de
				> aanduiding: Obx
				> En op de 2e positie kan je dan nog in theorie alle varianten vinden die op
				> het overzicht staan.
				>
				> Wat ik met "apart" bedoel, is dat we los van het gedrukte exemplaar een
				> beschrijving hebben gemaakt. Omdat we heel praktisch te werk zijn gegaan kom
				> jij nu in de problemen, want in de praktijk is vaak het volgende gebeurd:
				> - we hebben aan een bestaand Aax-record (Agx-record enz.) een URL gevoegd in
				> het exemplaarblok als we toegang hebben tot de full text.
				> Aax staat voor een gedrukte monografie.
				> Hetzelfde hebben we ook bij tijdschriften gedaan: aan een Abx-titel een URL
				> toegevoegd.
				>
				> Uit praktische overwegingen heb ik hiervoor gekozen. Mijn gedachte is: 1
				> record ongeacht of het gedrukt of electronisch is. -->
			</xsl:variable>

			<xsl:choose>
				<xsl:when test="$type='Aa' or $type='Oa' or $type='Ae' or $type='Oe' or $type='Af' or $type='Of' or $type='AF' or $type='OF' or $type='Ag' or $type='Og' or $type='Am' or $type='Om' or $type='AM' or $type='OM'">	
					<typeOfResource>text</typeOfResource>
					<genre>info:eu-repo/semantics/book</genre>
					<!-- het volgende verstoord de facetten -->
					<!--genre authority="marcgt">book</genre-->
				</xsl:when>
				<xsl:when test="$type='Ab'">
					<typeOfResource>text</typeOfResource>
					<xsl:variable name="test">
						<xsl:for-each select="datafield[@tag='209A']/subfield[@code='a']">
							<xsl:value-of select="substring-before(., ' ')"/>
							<xsl:text>|</xsl:text>
						</xsl:for-each>
					</xsl:variable>
					<xsl:choose>
						<xsl:when test="contains($test, 'T|') or contains($test, 'E|') or contains($test, 'TF|')">
							<genre authority="marcgt">journal</genre>
						</xsl:when>
						<xsl:otherwise>
							<genre authority="marcgt">series</genre>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:when>
				<xsl:when test="$type='Ac'">
					<typeOfResource>text</typeOfResource>
					<genre authority="marcgt">multivolume monograph</genre>
				</xsl:when>
				<!-- koepeltitel MP in reeks -->
				<xsl:when test="$type='Al'">
					<typeOfResource>text</typeOfResource>
					<genre authority="marcgt">series</genre>
				</xsl:when>
				<xsl:when test="$type='As'">
					<typeOfResource>text</typeOfResource>
					<genre>info:eu-repo/semantics/article</genre>
				</xsl:when>
				<xsl:when test="$type='Ba'">
					<genre>audiovisual</genre>
				</xsl:when>
				<xsl:when test="$type='Ma'">
					<typeOfResource>notated music</typeOfResource>
					<genre>sheet music</genre>
				</xsl:when>
				<xsl:when test="$type='Ob'">
					<typeOfResource>text</typeOfResource>
					<genre authority="marcgt">journal</genre>
				</xsl:when>
				<xsl:otherwise>
					<genre><xsl:value-of select="$type"/></genre>
				</xsl:otherwise>
			</xsl:choose>

			<!-- 002@/1="T" -> thesaurus record -->

			<!-- PPN -->
			<xsl:for-each select="datafield[@tag='003@']/subfield[@code=0]">
				<xsl:call-template name="recordInfo">
					<xsl:with-param name="ppn"><xsl:value-of select="."/></xsl:with-param>
				</xsl:call-template>
			</xsl:for-each>

			<!-- ISBN -->
			<xsl:for-each select="datafield[@tag='004A']|datafield[@tag='004B']">
				<xsl:if test="subfield[@code='0']">
					<!-- 10 -->
					<xsl:variable name="isbn" select="subfield[@code='0']"/>
					<identifier type="uri">
						<xsl:attribute name="displayLabel">
							<xsl:value-of select="substring($isbn,1,1)"/>
							<xsl:text>-</xsl:text>
							<xsl:value-of select="substring($isbn,2,3)"/>
							<xsl:text>-</xsl:text>
							<xsl:value-of select="substring($isbn,5,5)"/>
							<xsl:text>-</xsl:text>
							<xsl:value-of select="substring($isbn,10,1)"/>
                                                        <xsl:if test="subfield[@code='g']">
								<xsl:text> </xsl:text><xsl:value-of select="subfield[@code='g']"/>
							</xsl:if>
						</xsl:attribute>
						<xsl:text>URN:ISBN:</xsl:text><xsl:value-of select="$isbn"/>
					</identifier>
				</xsl:if>
				<xsl:if test="subfield[@code='A']">
					<!-- 13 -->
					<xsl:variable name="isbn" select="subfield[@code='A']"/>
					<identifier type="uri">
						<xsl:attribute name="displayLabel">
							<xsl:value-of select="substring($isbn,1,3)"/>
							<xsl:text>-</xsl:text>
							<xsl:value-of select="substring($isbn,4,1)"/>
							<xsl:text>-</xsl:text>
							<xsl:value-of select="substring($isbn,5,3)"/>
							<xsl:text>-</xsl:text>
							<xsl:value-of select="substring($isbn,8,5)"/>
							<xsl:text>-</xsl:text>
							<xsl:value-of select="substring($isbn,13,1)"/>
                                                        <xsl:if test="subfield[@code='g']">
								<xsl:text> </xsl:text><xsl:value-of select="subfield[@code='g']"/>
							</xsl:if>
						</xsl:attribute>
						<xsl:text>URN:ISBN:</xsl:text><xsl:value-of select="$isbn"/>
					</identifier>
				</xsl:if>
			</xsl:for-each>

			<!-- ISSN -->
			<xsl:for-each select="datafield[@tag='005A']/subfield[@code='0']">
				<identifier type="uri">
					<xsl:text>URN:ISSN:</xsl:text>
					<xsl:value-of select="."/>
				</identifier>
			</xsl:for-each>

			<!-- LANGUAGE -->
			<xsl:for-each select="datafield[@tag='010@']/subfield[@code='a']">
				<language>
					<languageTerm type="code" authority="rfc3066">
						<xsl:variable name="language">
							<xsl:value-of select="."/>
						</xsl:variable>
						<xsl:choose>
							<xsl:when test="$language='eng'">en</xsl:when>
							<xsl:when test="$language='ned'">nl</xsl:when>
							<xsl:when test="$language='dui'">de</xsl:when>
							<xsl:when test="$language='fra'">fr</xsl:when>
							<xsl:when test="$language='alb'">sq</xsl:when>
							<xsl:when test="$language='chi'">zh</xsl:when>
							<xsl:when test="$language='dee'">da</xsl:when>
							<xsl:when test="$language='esp'">eo</xsl:when>
							<xsl:when test="$language='ita'">it</xsl:when>
							<xsl:when test="$language='jap'">ja</xsl:when>
							<xsl:when test="$language='lat'">la</xsl:when>
							<xsl:when test="$language='noo'">no</xsl:when>
							<xsl:when test="$language='por'">pt</xsl:when>
							<xsl:when test="$language='rus'">ru</xsl:when>
							<xsl:when test="$language='spa'">es</xsl:when>
							<xsl:when test="$language='zwe'">sv</xsl:when>
							<!-- http://www.loc.gov/standards/iso639-2/php/code_list.php -->
							<!--table language =
							"afr" "afr",  "alb" "alb",  "ang" "ang",  "ara" "ara",  "ars" "arc",
							"arm" "arm",  "aze" "aze",  "bli" "ban",  "bas" "baq",  "bre" "bre",
							"bul" "bul",  "cat" "cat",  "che" "che",  "chi" "chi",  "cor" "cor",
							"dee" "dan",  "dui" "ger",  "egy" "egy",  "eng" "eng",  "esp" "esp",
							"est" "est",  "far" "far",  "fin" "fin",  "fra" "fre",  "fri" "fri",
							"gae" "gae",  "geo" "geo",  "grn" "gre",  "heb" "heb",  "hin" "hin",
							"hon" "hun",  "ier" "iri",  "ind" "ind",  "ita" "ita",  "jap" "jpn",
							"jav" "jav",  "jid" "yid",  "kak" "cau",  "kaz" "kaz",  "slk" "chu",
							"kir" "kir",  "kur" "kur",  "kop" "cop",  "lap" "lap",  "lat" "lat",
							"let" "lat",  "lit" "lit",  "mac" "mac",  "mly" "mal",  "mal" "may",
							"mlt" "mlt",  "min" "min",  "mol" "mol",  "ned" "dut",  "noo" "nor",
							"oei" "uig",  "oek" "ukr",  "oes" "uzb",  "oss" "oss",  "pap" "pap",
							"ppa" "paa",  "poo" "pol",  "por" "por",  "roe" "rum",  "rus" "rus",
							"scc" "scc",  "scr" "scr",  "slo" "slo",  "slw" "slw",  "spa" "spa",
							"tse" "cze",  "tsj" "chv",  "tuk" "tuk",  "tur" "tur",  "mis" "mis",
							"wel" "wel",  "wit" "bel",  "ijs" "ice",  "mul" "mul",  "zig" "rom",
							"zwe" "swe",
							default "und";-->

							<xsl:otherwise><xsl:value-of select="."/></xsl:otherwise>
						</xsl:choose>
					</languageTerm>
				</language>
			</xsl:for-each>

			<!-- ORIGININFO -->
			<xsl:if test="datafield[@tag='011@'] or datafield[@tag='032@'] or datafield[@tag='033A']">
				<originInfo>
					<xsl:if test="datafield[@tag='011@']">
						<dateIssued>
							<xsl:choose>
								<xsl:when test="datafield[@tag='011@']/subfield[@code='a']">
									<!-- start date -->
									<xsl:value-of select="datafield[@tag='011@']/subfield[@code='a']"/>
									<xsl:if test="datafield[@tag='011@']/subfield[@code='b']">
										<xsl:text>-</xsl:text>
										<xsl:value-of select="datafield[@tag='011@']/subfield[@code='b']"/>
									</xsl:if>
									<xsl:if test="datafield[@tag='011@']/subfield[@code='n']">
										<xsl:text> (</xsl:text>
										<xsl:value-of select="datafield[@tag='011@']/subfield[@code='n']"/>
                                                                                <xsl:text>)</xsl:text>
									</xsl:if>
								</xsl:when>
								<xsl:otherwise>
									<!-- 1940-.... -->
									<xsl:value-of select="datafield[@tag='011@']/subfield[@code='n']"/>
								</xsl:otherwise>
							</xsl:choose>
						</dateIssued>
					</xsl:if>
					<xsl:for-each select="datafield[@tag='032@']">
						<edition>
							<xsl:value-of select="subfield[@code='a']"/>
							<xsl:if test="subfield[@code='c']"><xsl:text> </xsl:text></xsl:if>
							<xsl:value-of select="subfield[@code='c']"/>
						</edition>
					</xsl:for-each>
					<xsl:for-each select="datafield[@tag='033A']">
						<xsl:if test="subfield[@code='n']">
							<publisher>
								<xsl:value-of select="subfield[@code='n']"/>
							</publisher>
						</xsl:if>
						<xsl:if test="subfield[@code='p']">
							<place>
								<xsl:for-each select="subfield[@code='p']">
									<placeTerm type="text">
										<xsl:value-of select="."/>
									</placeTerm>
								</xsl:for-each>
							</place>
						</xsl:if>
					</xsl:for-each>
				</originInfo>
			</xsl:if>

			<!-- NOTE -->
			<xsl:for-each select="datafield[@tag='020I']/subfield[@code='a']">
				<note><xsl:value-of select="."/></note>
			</xsl:for-each>
			<xsl:for-each select="datafield[@tag='037A']/subfield[@code='a']">
				<note><xsl:value-of select="."/></note>
			</xsl:for-each>
			<xsl:for-each select="datafield[@tag='039B']">
				<note>
					<xsl:value-of select="./subfield[@code='a']"/>
					<xsl:text>: </xsl:text>
					<xsl:value-of select="./subfield[@code='c']"/>
				</note>
			</xsl:for-each>
			<xsl:for-each select="datafield[@tag='037E']/subfield[@code='a']">
				<note><xsl:value-of select="."/></note>
			</xsl:for-each>
			<xsl:for-each select="datafield[@tag='047C']/subfield[@code='a']">
				<note><xsl:value-of select="."/></note>
			</xsl:for-each>

			<!-- TITLE -->
			<xsl:for-each select="datafield[@tag='021A']">
				<xsl:if test="subfield/@code='a'">
					<xsl:call-template name="title">
						<xsl:with-param name="code">a</xsl:with-param>
					</xsl:call-template>
				</xsl:if>
			</xsl:for-each>
			<!-- ONZELFSTANDIG DEEL van een meerdelige publicatie -->
			<xsl:if test="$type='Ae' or $type='Af' or $type='Of'">
				<xsl:variable name="part">
					<xsl:for-each select="datafield[@tag='021B']">
						<xsl:if test="subfield[@code='l']|subfield[@code='m']">
							<xsl:if test="subfield[@code='l']">
								<xsl:value-of select="subfield[@code='l']"/>
							</xsl:if>
							<xsl:if test="subfield[@code='m']">
								<xsl:text> </xsl:text>
								<xsl:value-of select="subfield[@code='m']"/>
							</xsl:if>
						</xsl:if>
					</xsl:for-each>
				</xsl:variable>
				<xsl:for-each select="datafield[@tag='021A']">
					<xsl:call-template name="title">
						<xsl:with-param name="code">8</xsl:with-param>
						<xsl:with-param name="part" select="$part"/>
					</xsl:call-template>
						<!--xsl:if test="subfield/@code='9'">
							<xsl:call-template name="recordInfo">
								<xsl:with-param name="ppn"><xsl:value-of select="subfield[@code='9']"/></xsl:with-param>
							</xsl:call-template>
						</xsl:if-->
				</xsl:for-each>
			</xsl:if>

			<!-- AUTHOR -->
			<xsl:for-each select="datafield[@tag='028A']">
				<!-- primaire auteur -->
				<xsl:call-template name="author">
					<xsl:with-param name="role">aut</xsl:with-param>
				</xsl:call-template>
			</xsl:for-each>
			<xsl:for-each select="datafield[@tag='028B']">
				<!-- coauteur -->
				<xsl:call-template name="author">
					<xsl:with-param name="role">aut</xsl:with-param>
				</xsl:call-template>
			</xsl:for-each>
			<xsl:for-each select="datafield[@tag='028C']">
				<!-- secundaire auteur -->
				<xsl:choose>
					<xsl:when test="$type='Af' or $type='Of'">
						<!-- onzelfstandig deel van een meerdelige publicatie -->
						<xsl:call-template name="author">
							<xsl:with-param name="role">aut</xsl:with-param>
						</xsl:call-template>
					<!-- ook 028D ?? -->
					</xsl:when>
					<xsl:otherwise>
						<xsl:call-template name="author">
							<xsl:with-param name="role">clb</xsl:with-param>
						</xsl:call-template>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:for-each>

			<physicalDescription>
				<xsl:choose>
					<xsl:when test="starts-with($type,'A')">
						<form authority="marcform">print</form>
					</xsl:when>
					<xsl:when test="starts-with($type,'O')">
						<form authority="marcform">electronic</form>
					</xsl:when>
				</xsl:choose>
				<xsl:if test="datafield[@tag='034D'] or datafield[@tag='034I'] or datafield[@tag='034M']">
					<extent>
						<xsl:if test="datafield[@tag='034D']">
							<xsl:value-of select="datafield[@tag='034D']/subfield[@code='a']"/>
						</xsl:if>
						<xsl:if test="datafield[@tag='034I']">
							<xsl:if test="datafield[@tag='034D']"><xsl:text>; </xsl:text></xsl:if>
							<xsl:value-of select="datafield[@tag='034I']/subfield[@code='a']"/>
						</xsl:if>
						<xsl:if test="datafield[@tag='034M']">
							<xsl:if test="datafield[@tag='034D'] or datafield[@tag='034I']"><xsl:text>; </xsl:text></xsl:if>
							<xsl:value-of select="datafield[@tag='034M']/subfield[@code='a']"/>
						</xsl:if>
						<xsl:if test="datafield[@tag='034K']">
							<xsl:if test="datafield[@tag='034D'] or datafield[@tag='034I'] or datafield[@tag='034M']"><xsl:text>; </xsl:text></xsl:if>
							<xsl:value-of select="datafield[@tag='034K']/subfield[@code='a']"/>
						</xsl:if>
					</extent>
				</xsl:if>
			</physicalDescription>

			<!-- SERIES -->
			<!-- alleen 036F gebruiken, 036E is redundant -->
			<!-- xsl:for-each select="datafield[@tag='036E']">
				<relatedItem type="series">
					<xsl:call-template name="title">
						<xsl:with-param name="code">a</xsl:with-param>
					</xsl:call-template>
				</relatedItem>
			</xsl:for-each -->
			<xsl:for-each select="datafield[@tag='036F']">
				<relatedItem type="series">
					<xsl:call-template name="title">
						<xsl:with-param name="code">8</xsl:with-param>
					</xsl:call-template>
					<xsl:if test="subfield[@code='9']">
						<xsl:call-template name="recordInfo">
							<xsl:with-param name="ppn"><xsl:value-of select="subfield[@code='9']"/></xsl:with-param>
						</xsl:call-template>
					</xsl:if>
					<xsl:if test="subfield[@code='l']">
						<part>
							<detail>
								<number><xsl:value-of select="subfield[@code='l']"/></number>
							</detail>
						</part>
					</xsl:if>
				</relatedItem>
			</xsl:for-each>


			<!-- 040@ persoon als onderwerp? -->

			<!-- SUBJECT -->
			<xsl:if test="datafield[@tag='044K'] or datafield[@tag='044N'] or datafield[@tag='144Z']">
				<subject>
					<xsl:for-each select="datafield[@tag='044K']">
						<topic>
							<xsl:value-of select="subfield[@code='a']"/>
							<xsl:value-of select="translate(subfield[@code='8'],'$','')"/>
						</topic>
					</xsl:for-each>
					<xsl:for-each select="datafield[@tag='044N']">
						<topic>
							<xsl:value-of select="subfield[@code='8']"/>
						</topic>
					</xsl:for-each>
					<xsl:for-each select="datafield[@tag='144Z']">
						<topic>
							<xsl:value-of select="subfield[@code='a']"/>
						</topic>
					</xsl:for-each>
				</subject>
			</xsl:if>

			<!-- CLASSIFICATION -->
			<xsl:for-each select="datafield[@tag='045Q']/subfield[@code='8']">
				<classification>
					<xsl:value-of select="."/>
				</classification>
			</xsl:for-each>

			<xsl:for-each select="datafield[@tag='047H']">
				<relatedItem type="isReferencedBy">
					<xsl:call-template name="title">
						<xsl:with-param name="code">a</xsl:with-param>
					</xsl:call-template>
				</relatedItem>
			</xsl:for-each>

			<!-- LOCATION -->
			<xsl:if test="datafield[@tag='009P']/subfield[@code='a'] or datafield[@tag='201B']">
				<location>
					<xsl:for-each select="datafield[@tag='009P']/subfield[@code='a']">
						<url><xsl:value-of select="."/></url>
					</xsl:for-each>
					<holdingSimple>
						<xsl:for-each select="datafield[@tag='201B']">
							<copyInformation>
								<xsl:variable name="occurrence" select="@occurrence"/>
								<!-- form -->
								<!-- subLocation -->
								<xsl:for-each select="../datafield[@tag='209A'][@occurrence=$occurrence]">
									<xsl:if test="subfield[@code='x']='00'">
										<shelfLocator>
											<xsl:value-of select="subfield[@code='a']"/>
										</shelfLocator>
									</xsl:if>
								</xsl:for-each>
								<xsl:for-each select="../datafield[@tag='209R'][@occurrence=$occurrence]/subfield[@code='a']">
									<electronicLocator><xsl:value-of select="."/></electronicLocator>
								</xsl:for-each>
								<note type="occurrence">
									<xsl:value-of select="$occurrence"/>
								</note>
								<note type="epn">
									<xsl:value-of select="../datafield[@tag='203@'][@occurrence=$occurrence]/subfield[@code='0']"/>
								</note>
								<xsl:for-each select="../datafield[@tag='209A'][@occurrence=$occurrence]">
									<xsl:if test="subfield[@code='x']='00'">
										<note type="status">
											<xsl:value-of select="subfield[@code='d']"/>
										</note>
									</xsl:if>
								</xsl:for-each>
								<xsl:if test="../datafield[@tag='231@'][@occurrence=$occurrence]">
									<xsl:variable name="holding">
										<xsl:call-template name="holding">
											<xsl:with-param name="occurrence" select="@occurrence"/>
										</xsl:call-template>
									</xsl:variable>
									<xsl:if test="string-length($holding)&gt;0">
										<enumerationAndChronology unitType="1">
											<xsl:value-of select="translate($holding,'^','')"/>
										</enumerationAndChronology>
									</xsl:if>
									<!-- supplements, indices, collection -->
									<!-- unitType="2" ==> supplement
									 	 unitType="3" ==> index -->
									<xsl:for-each select="../datafield[@tag='231@'][@occurrence=$occurrence]/subfield[@code='4']">
										<enumerationAndChronology unitType="3">
											<xsl:value-of select="."/>
										</enumerationAndChronology>
									</xsl:for-each>
								</xsl:if>
								<xsl:for-each select="../datafield[@tag='237A'][@occurrence=$occurrence]/subfield[@code='a']">
									<note><xsl:value-of select="."/></note>
								</xsl:for-each>
							</copyInformation>
						</xsl:for-each>
					</holdingSimple>
					<xsl:if test="datafield[@tag='231@']">
						<holdingExternal>
							<a:copies>
								<xsl:for-each select="datafield[@tag='231@']">
									<xsl:variable name="occurrence" select="@occurrence"/>
									<a:copy>
										<a:occurrence>
											<xsl:value-of select="$occurrence"/>
										</a:occurrence>
										<a:epn>
											<xsl:value-of select="../datafield[@tag='203@'][@occurrence=$occurrence]/subfield[@code='0']"/>
										</a:epn>
										<xsl:for-each select="../datafield[@tag='231@'][@occurrence=$occurrence]/subfield[@code='4']">
											<!-- dit veld bevat vaak, maar niet alleen en niet altijd collectie -->
										<enumerationAndChronology unitType="3">
											<xsl:value-of select="."/>
										</enumerationAndChronology>
										</xsl:for-each>
										<xsl:for-each select="../datafield[@tag='209A'][@occurrence=$occurrence]/subfield[@code='a']">
											<shelfLocator>
												<xsl:value-of select="."/>
											</shelfLocator>
										</xsl:for-each>
										<xsl:for-each select="../datafield[@tag='209A'][@occurrence=$occurrence]/subfield[@code='d']">
											<a:status>
												<xsl:value-of select="."/>
											</a:status>
										</xsl:for-each>
										<xsl:variable name="holding">
											<xsl:call-template name="holding">
												<xsl:with-param name="occurrence" select="@occurrence"/>
											</xsl:call-template>
										</xsl:variable>
										<xsl:if test="string-length($holding)&gt;0">
											<xsl:call-template name="threshold">
												<xsl:with-param name="holding" select="$holding"/>
											</xsl:call-template>
										</xsl:if>
									</a:copy>
								</xsl:for-each>
							</a:copies>
						</holdingExternal>
					</xsl:if>
				</location>
			</xsl:if>
		</mods>
	</xsl:template>

	<!-- TITLE -->
	<xsl:template name="title">
		<xsl:param name="code"/>
		<xsl:param name="part"/>
		<xsl:variable name="title">
			<xsl:value-of select="subfield[@code=$code]"/>
		</xsl:variable>
		<xsl:variable name="titleNonSort">
			<xsl:choose>
				<xsl:when test="contains($title,'@')">
					<xsl:variable name="before">
						<xsl:value-of select="substring-before($title,'@')"/>
					</xsl:variable>
					<xsl:variable name="after">
						<xsl:value-of select="substring-after($title,'@')"/>
					</xsl:variable>
					<xsl:value-of select="concat($before,$after)"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$title"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<titleInfo>
			<title>
				<xsl:choose>
					<!-- als titel uit 021A$8 komt dan wordt ISBD gebruikt:
						"[titel] / [auteur]" -->
					<xsl:when test="contains($titleNonSort,' / ')">
						<xsl:value-of select="substring-before($titleNonSort,' / ')"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$titleNonSort"/>
					</xsl:otherwise>
				</xsl:choose>
				<xsl:if test="string-length($part)&gt; 0">
					<xsl:text> (</xsl:text><xsl:value-of select="$part"/><xsl:text>)</xsl:text>
				</xsl:if>
				<xsl:if test="../datafield[@tag='009P']">
					<xsl:text> [Online Resource]</xsl:text>
				</xsl:if>
			</title>
			<xsl:for-each select="subfield[@code='d']">
				<subTitle>
					<xsl:value-of select="."/>
				</subTitle>
			</xsl:for-each>
		</titleInfo>
	</xsl:template>

	<!-- AUTHOR -->
	<xsl:template name="author">
		<xsl:param name="role"/>
		<name type="personal">
			<namePart type="family">
				<xsl:value-of select="subfield[@code='a']"/>
				<xsl:if test="subfield[@code='c']">
					<xsl:text>, </xsl:text>
				</xsl:if>
				<xsl:value-of select="subfield[@code='c']"/>
			</namePart>
			<xsl:if test="subfield[@code='d']">
				<namePart type="given">
					<xsl:value-of select="subfield[@code='d']"/>
				</namePart>
			</xsl:if>
			<role>
				<roleTerm authority="marcrelator" type="code"><xsl:value-of select="$role"/></roleTerm>
			</role>
		</name>
	</xsl:template>

	<!-- RECORDINFO -->
	<xsl:template name="recordInfo">
		<xsl:param name="ppn"/>
		<recordInfo>
			<recordIdentifier><xsl:value-of select="$ppn"/></recordIdentifier>
			<recordContentSource>GGC</recordContentSource>
		</recordInfo>
	</xsl:template>

	<!-- HYPEN -->
	<xsl:template name="hyphen">
		<xsl:param name="position"/>
		<xsl:if test="../subfield[position()=$position][@code='o' or @code='n' or @code='k']">
			<xsl:text> -</xsl:text>
		</xsl:if>
	</xsl:template>

	<!-- HOLDING -->
	<xsl:template name="holding">
		<xsl:param name="occurrence"/>
		<xsl:for-each select="../datafield[@tag='231@'][@occurrence=$occurrence]/subfield">
			<xsl:variable name="position" select="position()"/>
			<xsl:choose>
				<!--
				*         $b dag
				*         $c maand
				*         $d volume
				*         $e aflevering
				*         $f serieaanduiding
				*         $g commentaar
				*         $j beginjaar
				*         $k eindjaar
				*         $l dag
				*         $m maand
				*         $n volume
				*         $o aflevering
				*         $p serieaanduiding
				*         $q commentaar
				*         $1 bewaartermijn
				*         $2 beginjaar bij bewaartermijn
				*         $3 aantal bewaarde uitgaven
				*         $4 commentaar over gehele holding
				*         $5 aanduiding parallelle nummering
				*         $6 aanduiding lopende holding
				-->
				<!-- issue -->
				<xsl:when test="@code='e'">
					<xsl:text> issue </xsl:text>
					<xsl:value-of select="."/>
					<xsl:text>^</xsl:text>
					<xsl:call-template name="hyphen">
						<xsl:with-param name="position" select="$position+1"/>
					</xsl:call-template>
				</xsl:when>
				<!-- volume -->
				<xsl:when test="@code='d'">
					<xsl:text>vol. </xsl:text>
					<xsl:value-of select="."/>
					<xsl:text>^</xsl:text>
					<xsl:call-template name="hyphen">
						<xsl:with-param name="position" select="$position+1"/>
					</xsl:call-template>
				</xsl:when>
				<!-- year -->
				<xsl:when test="@code='j'">
					<xsl:text> (</xsl:text>
					<xsl:value-of select="."/>
					<xsl:text>)</xsl:text>
					<xsl:call-template name="hyphen">
						<xsl:with-param name="position" select="$position+1"/>
					</xsl:call-template>
				</xsl:when>
				<!-- week -->
				<xsl:when test="@code='g'">
					<xsl:text> </xsl:text>
					<xsl:value-of select="."/>
					<xsl:call-template name="hyphen">
						<xsl:with-param name="position" select="$position+1"/>
					</xsl:call-template>
				</xsl:when>
				<!-- month -->
				<xsl:when test="@code='m'">
					<xsl:text> </xsl:text>
					<xsl:value-of select="."/>
					<xsl:call-template name="hyphen">
						<xsl:with-param name="position" select="$position+1"/>
					</xsl:call-template>
				</xsl:when>
				<!-- continuing -->
				<xsl:when test="@code='6'">
					<xsl:text> - ...</xsl:text>
				</xsl:when>
				<!-- issue -->
				<xsl:when test="@code='o'">
					<xsl:text> issue </xsl:text>
					<xsl:value-of select="."/>
					<xsl:text>^</xsl:text>
					<xsl:text> </xsl:text>
				</xsl:when>
				<!-- volume -->
				<xsl:when test="@code='n'">
					<xsl:text> vol. </xsl:text>
					<xsl:value-of select="."/>
					<xsl:text>^</xsl:text>
					<xsl:text> </xsl:text>
				</xsl:when>
				<!-- year -->
				<xsl:when test="@code='k'">
					<xsl:text> (</xsl:text>
					<xsl:value-of select="."/>
					<xsl:text>)</xsl:text>
				</xsl:when>
				<!-- week -->
				<xsl:when test="@code='q'">
					<xsl:text> </xsl:text>
					<xsl:value-of select="."/>
				</xsl:when>
				<!-- month -->
				<!-- encore -->
				<xsl:when test="@code='0'">
					<xsl:text>; </xsl:text>
				</xsl:when>
			</xsl:choose>
		</xsl:for-each>
	</xsl:template>

	<!-- THRESHOLD -->
	<xsl:template name="threshold">
		<xsl:param name="holding"/>
		<xsl:comment><xsl:value-of select="$holding"/></xsl:comment>
		<xsl:variable name="threshold">
			<xsl:choose>
				<xsl:when test="contains($holding,';')">
					<xsl:value-of select="substring-before($holding,';')"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$holding"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="other">
			<xsl:if test="contains($holding,';')">
				<xsl:value-of select="substring-after($holding,';')"/>
			</xsl:if>
		</xsl:variable>
		<xsl:variable name="lower">
			<xsl:choose>
				<xsl:when test="contains($threshold,' - ')">
					<xsl:value-of select="substring-before($threshold,' - ')"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$threshold"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="lvolume">
			<xsl:value-of select="substring-before(substring-after($lower,'vol. '),'^')"/>
		</xsl:variable>
		<xsl:variable name="lissue" select="substring-before(substring-after($lower,'issue '),'^')"/>
		<xsl:variable name="ldate">
			<xsl:choose>
				<xsl:when test="contains(substring-before(substring-after($lower,'('),')'),'/')">
					<xsl:value-of select="substring-before(substring-before(substring-after($lower,'('),')'),'/')"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="substring-before(substring-after($lower,'('),')')"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="upper" select="substring-after($threshold,' - ')"/>
		<xsl:variable name="uvolume" select="substring-before(substring-after($upper,'vol. '),'^')"/>
		<xsl:variable name="uissue" select="substring-before(substring-after($upper,'issue '),'^')"/>
		<xsl:variable name="udate">
			<xsl:choose>
				<xsl:when test="contains(substring-before(substring-after($upper,'('),')'),'/')">
					<xsl:choose>
						<xsl:when test="string-length(substring-after(substring-before(substring-after($upper,'('),')'),'/'))=4">
							<xsl:value-of select="substring-after(substring-before(substring-after($upper,'('),')'),'/')"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:variable name="h" select="substring-before(substring-after($upper,'('),')')"/>
							<!-- maak netjes van 1999/00 1999/2000 -->
							<xsl:variable name="h1" select="substring-before($h,'/')"/>
							<xsl:variable name="e" select="concat(substring($h,1,2),substring-after($h,'/'))"/>
							<xsl:choose>
								<xsl:when test="$e &lt; $h1">
									<xsl:value-of select="$e + 100"/>
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of select="$e"/>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="substring-before(substring-after($upper,'('),')')"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<a:range>
			<xsl:if test="string-length($lvolume)&gt;0">
				<a:threshold>
					<a:attribute>volume</a:attribute>
					<a:operator>ge</a:operator>
					<a:value>
						<xsl:value-of select="$lvolume"/>
					</a:value>
					<xsl:if test="string-length($lissue)&gt;0">
						<a:threshold>
							<a:attribute>issue</a:attribute>
							<a:operator>ge</a:operator>
							<a:value>
								<xsl:value-of select="$lissue"/>
							</a:value>
						</a:threshold>
					</xsl:if>
				</a:threshold>
			</xsl:if>
			<xsl:if test="string-length($ldate)&gt;0">
				<a:threshold>
					<a:attribute>date</a:attribute>
					<a:operator>ge</a:operator>
					<a:value>
						<xsl:value-of select="$ldate"/>
					</a:value>
				</a:threshold>
			</xsl:if>
			<xsl:choose>
				<xsl:when test="string-length($upper)&gt;0 and $upper != '...'">
					<xsl:choose>
						<xsl:when test="string-length($uvolume)&gt;0">
						    <a:threshold>
							    <a:attribute>volume</a:attribute>
					    		<a:operator>le</a:operator>
						    	<a:value>
							    	<xsl:value-of select="$uvolume"/>
					    		</a:value>
						    	<xsl:if test="string-length($uissue)&gt;0">
							    	<a:threshold>
								    	<a:attribute>issue</a:attribute>
									    <a:operator>le</a:operator>
									    <a:value>
										    <xsl:value-of select="$uissue"/>
									    </a:value>
								    </a:threshold>
							    </xsl:if>
						    </a:threshold>
						</xsl:when>
				 		<xsl:when test="string-length($uissue)&gt;0">
				 			<!-- issue but no volume -->
							<a:threshold>
				 				<xsl:if test="string-length($lvolume)&gt;0">
				 					<!-- upper volume = lower volume -->
				 					<a:attribute>volume</a:attribute>
				 					<a:operator>le</a:operator>
				 					<a:value>
				 						<xsl:value-of select="$lvolume"/>
				 					</a:value>
				 				</xsl:if>
								<xsl:if test="string-length($uissue)&gt;0">
									<a:threshold>
										<a:attribute>issue</a:attribute>
										<a:operator>le</a:operator>
										<a:value>
											<xsl:value-of select="$uissue"/>
										</a:value>
									</a:threshold>
								</xsl:if>
							</a:threshold>
						</xsl:when>
					</xsl:choose>
					<xsl:choose>
	   					<xsl:when test="string-length($udate)&gt;0">
							<a:threshold>
								<a:attribute>date</a:attribute>
								<a:operator>le</a:operator>
					  			<a:value>
						 			<xsl:value-of select="$udate"/>
							</a:value>
						    </a:threshold>
						</xsl:when>
				 		<xsl:when test="string-length($ldate)&gt;0">
				 			<a:threshold>
				 				<a:attribute>date</a:attribute>
				 				<a:operator>le</a:operator>
				 				<a:value>
									<xsl:value-of select="$ldate"/>
				 				</a:value>
							</a:threshold>
						</xsl:when>
					</xsl:choose>
				</xsl:when>
				<xsl:otherwise>
					<xsl:if test="string-length($upper)=0 or $upper != '...'">
						<!-- there is no upper ==> upper = lower -->
						<xsl:if test="string-length($lvolume)&gt;0">
							<a:threshold>
								<a:attribute>volume</a:attribute>
								<a:operator>le</a:operator>
								<a:value>
									<xsl:value-of select="$lvolume"/>
								</a:value>
								<xsl:if test="string-length($lissue)&gt;0">
									<a:threshold>
										<a:attribute>issue</a:attribute>
										<a:operator>le</a:operator>
										<a:value>
											<xsl:value-of select="$lissue"/>
										</a:value>
									</a:threshold>
								</xsl:if>
							</a:threshold>
						</xsl:if>
						<xsl:if test="string-length($ldate)&gt;0">
							<a:threshold>
								<a:attribute>date</a:attribute>
								<a:operator>le</a:operator>
								<a:value>
									<xsl:value-of select="$ldate"/>
								</a:value>
							</a:threshold>
						</xsl:if>
					</xsl:if>
				</xsl:otherwise>
			</xsl:choose>
		</a:range>
		<xsl:if test="string-length($other)&gt;0">
			<xsl:call-template name="threshold">
				<xsl:with-param name="holding" select="$other"/>
			</xsl:call-template>
		</xsl:if>
	</xsl:template>
</xsl:stylesheet>
